@@ -9,34 +9,41 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include <minizinc/ast.hh>
+#include <minizinc/eval_par.hh>
 #include <minizinc/flat_exp.hh>
+#include <minizinc/flatten_internal.hh>
+#include <minizinc/gc.hh>
+
+#include <cassert>
+#include <vector>
 
 namespace MiniZinc {
 
 EE flatten_arrayaccess(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b) {
   CallStackItem _csi(env, e);
   EE ret;
-  auto* aa = e->cast<ArrayAccess>();
+  auto* aa = Expression::cast<ArrayAccess>(e);
   KeepAlive aa_ka = aa;
 
   Ctx nctx = ctx;
   nctx.b = +nctx.b;
   nctx.neg = false;
-  EE eev = flat_exp(env, nctx, aa->v(), nullptr, nullptr);
+  EE eev = flat_exp(env, nctx, aa->v(), nullptr, nctx.partialityVar(env));
   std::vector<EE> ees;
 
 start_flatten_arrayaccess:
   for (unsigned int i = 0; i < aa->idx().size(); i++) {
     Expression* tmp = follow_id_to_decl(aa->idx()[i]);
-    if (auto* vd = tmp->dynamicCast<VarDecl>()) {
+    if (auto* vd = Expression::dynamicCast<VarDecl>(tmp)) {
       tmp = vd->id();
     }
-    if (tmp->type().isPar()) {
+    if (Expression::type(tmp).isPar()) {
       ArrayLit* al;
-      if (eev.r()->isa<ArrayLit>()) {
-        al = eev.r()->cast<ArrayLit>();
+      if (Expression::isa<ArrayLit>(eev.r())) {
+        al = Expression::cast<ArrayLit>(eev.r());
       } else {
-        Id* id = eev.r()->cast<Id>();
+        Id* id = Expression::cast<Id>(eev.r());
         if (id->decl() == nullptr) {
           throw InternalError("undefined identifier");
         }
@@ -44,28 +51,38 @@ start_flatten_arrayaccess:
           throw InternalError("array without initialiser not supported");
         }
         Expression* id_e = follow_id(id);
-        if (id_e->isa<ArrayLit>()) {
-          al = id_e->cast<ArrayLit>();
+        if (Expression::isa<ArrayLit>(id_e)) {
+          al = Expression::cast<ArrayLit>(id_e);
         } else {
           throw InternalError("builtin function returning array not supported");
         }
       }
 
+      bool allAbsent = true;
+      bool anyAbsent = false;
       std::vector<KeepAlive> elems;
       std::vector<IntVal> idx(aa->idx().size());
-      std::vector<std::pair<int, int> > dims;
+      std::vector<std::pair<int, int>> dims;
       std::vector<Expression*> newaccess;
       std::vector<int> nonpar;
       std::vector<int> stack;
-      for (unsigned int j = 0; j < aa->idx().size(); j++) {
+      for (int j = 0; j < aa->idx().size(); j++) {
         Expression* tmp = follow_id_to_decl(aa->idx()[j]);
-        if (auto* vd = tmp->dynamicCast<VarDecl>()) {
+        if (auto* vd = Expression::dynamicCast<VarDecl>(tmp)) {
           tmp = vd->id();
         }
-        if (tmp->type().isPar()) {
+        if (Expression::type(tmp).isPar()) {
           GCLock lock;
-          idx[j] = eval_int(env, tmp).toInt();
+          auto* v = eval_par(env, tmp);
+          if (v == env.constants.absent) {
+            anyAbsent = true;
+            idx[j] = al->min(j);
+          } else {
+            allAbsent = false;
+            idx[j] = IntLit::v(Expression::cast<IntLit>(v));
+          }
         } else {
+          allAbsent = false;
           idx[j] = al->min(j);
           stack.push_back(static_cast<int>(nonpar.size()));
           nonpar.push_back(j);
@@ -74,19 +91,24 @@ start_flatten_arrayaccess:
         }
       }
       if (stack.empty()) {
-        bool success;
+        ArrayAccessSucess success;
         KeepAlive ka;
-        {
+        if (allAbsent) {
+          ka = env.constants.absent;
+        } else {
           GCLock lock;
           ka = eval_arrayaccess(env, al, idx, success);
-          if (!success && env.inMaybePartial == 0) {
-            ResultUndefinedError warning(env, al->loc(), "array access out of bounds");
+          if (anyAbsent) {
+            ka = env.constants.absent;
+          }
+          if (!success() && env.inMaybePartial == 0) {
+            ResultUndefinedError warning(env, Expression::loc(al), success.errorMessage(env, al));
           }
         }
-        ees.emplace_back(nullptr, constants().boollit(success));
+        ees.emplace_back(nullptr, env.constants.boollit(success()));
         ees.emplace_back(nullptr, eev.b());
         if (aa->type().isbool() && !aa->type().isOpt()) {
-          ret.b = bind(env, Ctx(), b, constants().literalTrue);
+          ret.b = bind(env, Ctx(), b, env.constants.literalTrue);
           ees.emplace_back(nullptr, ka());
           ret.r = conj(env, r, ctx, ees);
         } else {
@@ -101,17 +123,18 @@ start_flatten_arrayaccess:
           stack.pop_back();
           for (int i = al->min(nonpar[cur]); i <= al->max(nonpar[cur]); i++) {
             idx[nonpar[cur]] = i;
-            bool success;
+            ArrayAccessSucess success;
             GCLock lock;
             Expression* al_idx = eval_arrayaccess(env, al, idx, success);
-            if (!success) {
+            if (!success()) {
               if (env.inMaybePartial == 0) {
-                ResultUndefinedError warning(env, al->loc(), "array access out of bounds");
+                ResultUndefinedError warning(env, Expression::loc(al),
+                                             success.errorMessage(env, al));
               }
-              ees.emplace_back(nullptr, constants().literalFalse);
+              ees.emplace_back(nullptr, env.constants.literalFalse);
               ees.emplace_back(nullptr, eev.b());
               if (aa->type().isbool() && !aa->type().isOpt()) {
-                ret.b = bind(env, Ctx(), b, constants().literalTrue);
+                ret.b = bind(env, Ctx(), b, env.constants.literalTrue);
                 ret.r = conj(env, r, ctx, ees);
               } else {
                 ret.b = conj(env, b, ctx, ees);
@@ -127,7 +150,7 @@ start_flatten_arrayaccess:
             stack.pop_back();
           } else {
             idx[nonpar[cur]]++;
-            for (unsigned int j = cur + 1; j < nonpar.size(); j++) {
+            for (int j = cur + 1; j < nonpar.size(); j++) {
               stack.push_back(j);
             }
           }
@@ -139,12 +162,11 @@ start_flatten_arrayaccess:
       }
       {
         GCLock lock;
-        Expression* newal = new ArrayLit(al->loc(), elems_e, dims);
-        Type t = al->type();
-        t.dim(static_cast<int>(dims.size()));
-        newal->type(t);
+        Expression* newal = new ArrayLit(Expression::loc(al), elems_e, dims);
+        Expression::type(
+            newal, Type::arrType(env, Type::partop(static_cast<int>(dims.size())), al->type()));
         eev.r = newal;
-        auto* n_aa = new ArrayAccess(aa->loc(), newal, newaccess);
+        auto* n_aa = new ArrayAccess(Expression::loc(aa), newal, newaccess);
         n_aa->type(aa->type());
         aa = n_aa;
         aa_ka = aa;
@@ -152,24 +174,24 @@ start_flatten_arrayaccess:
     }
   }
 
-  if (aa->idx().size() == 1 && aa->idx()[0]->isa<ArrayAccess>()) {
-    auto* aa_inner = aa->idx()[0]->cast<ArrayAccess>();
+  if (aa->idx().size() == 1 && Expression::isa<ArrayAccess>(aa->idx()[0])) {
+    auto* aa_inner = Expression::cast<ArrayAccess>(aa->idx()[0]);
     ArrayLit* al;
-    if (eev.r()->isa<ArrayLit>()) {
-      al = eev.r()->cast<ArrayLit>();
+    if (Expression::isa<ArrayLit>(eev.r())) {
+      al = Expression::cast<ArrayLit>(eev.r());
     } else {
-      Id* id = eev.r()->cast<Id>();
+      Id* id = Expression::cast<Id>(eev.r());
       if (id->decl() == nullptr) {
         throw InternalError("undefined identifier");
       }
       if (id->decl()->e() == nullptr) {
         throw InternalError("array without initialiser not supported");
       }
-      al = follow_id(id)->cast<ArrayLit>();
+      al = Expression::cast<ArrayLit>(follow_id(id));
     }
-    if (aa_inner->v()->type().isPar()) {
+    if (Expression::type(aa_inner->v()).isPar()) {
       KeepAlive ka_al_inner = flat_cv_exp(env, ctx, aa_inner->v());
-      auto* al_inner = ka_al_inner()->cast<ArrayLit>();
+      auto* al_inner = Expression::cast<ArrayLit>(ka_al_inner());
       std::vector<Expression*> composed_e(al_inner->size());
       for (unsigned int i = 0; i < al_inner->size(); i++) {
         GCLock lock;
@@ -179,18 +201,18 @@ start_flatten_arrayaccess:
         }
         composed_e[i] = (*al)[static_cast<int>(inner_idx.toInt()) - al->min(0)];
       }
-      std::vector<std::pair<int, int> > dims(al_inner->dims());
+      std::vector<std::pair<int, int>> dims(al_inner->dims());
       for (int i = 0; i < al_inner->dims(); i++) {
         dims[i] = std::make_pair(al_inner->min(i), al_inner->max(i));
       }
       {
         GCLock lock;
-        Expression* newal = new ArrayLit(al->loc(), composed_e, dims);
+        Expression* newal = new ArrayLit(Expression::loc(al), composed_e, dims);
         Type t = al->type();
         t.dim(static_cast<int>(dims.size()));
-        newal->type(t);
+        Expression::type(newal, t);
         eev.r = newal;
-        auto* n_aa = new ArrayAccess(aa->loc(), newal, aa_inner->idx());
+        auto* n_aa = new ArrayAccess(Expression::loc(aa), newal, aa_inner->idx());
         n_aa->type(aa->type());
         aa = n_aa;
         aa_ka = aa;
@@ -203,16 +225,16 @@ flatten_arrayaccess:
   dimctx.neg = false;
   for (unsigned int i = 0; i < aa->idx().size(); i++) {
     Expression* tmp = follow_id_to_decl(aa->idx()[i]);
-    if (auto* vd = tmp->dynamicCast<VarDecl>()) {
+    if (auto* vd = Expression::dynamicCast<VarDecl>(tmp)) {
       tmp = vd->id();
     }
-    ees.push_back(flat_exp(env, dimctx, tmp, nullptr, nullptr));
+    ees.push_back(flat_exp(env, dimctx, tmp, nullptr, dimctx.partialityVar(env)));
   }
   ees.emplace_back(nullptr, eev.b());
 
   bool parAccess = true;
   for (unsigned int i = 0; i < aa->idx().size(); i++) {
-    if (!ees[i].r()->type().isPar()) {
+    if (!Expression::type(ees[i].r()).isPar()) {
       parAccess = false;
       break;
     }
@@ -220,40 +242,95 @@ flatten_arrayaccess:
 
   if (parAccess) {
     ArrayLit* al;
-    if (eev.r()->isa<ArrayLit>()) {
-      al = eev.r()->cast<ArrayLit>();
+    if (Expression::isa<ArrayLit>(eev.r())) {
+      al = Expression::cast<ArrayLit>(eev.r());
     } else {
-      Id* id = eev.r()->cast<Id>();
+      Id* id = Expression::cast<Id>(eev.r());
       if (id->decl() == nullptr) {
         throw InternalError("undefined identifier");
       }
       if (id->decl()->e() == nullptr) {
         throw InternalError("array without initialiser not supported");
       }
-      al = follow_id(id)->cast<ArrayLit>();
+      al = Expression::cast<ArrayLit>(follow_id(id));
     }
     KeepAlive ka;
-    bool success;
+    ArrayAccessSucess success;
+    bool allAbsent = true;
+    bool anyAbsent = false;
     {
       GCLock lock;
       std::vector<IntVal> dims(aa->idx().size());
       for (unsigned int i = aa->idx().size(); (i--) != 0U;) {
-        dims[i] = eval_int(env, ees[i].r());
+        auto* v = eval_par(env, ees[i].r());
+        if (v == env.constants.absent) {
+          anyAbsent = true;
+          dims[i] = al->min(i);
+        } else {
+          allAbsent = false;
+          dims[i] = IntLit::v(Expression::cast<IntLit>(v));
+        }
+      }
+      if (allAbsent) {
+        ka = env.constants.absent;
+      } else {
+        ka = eval_arrayaccess(env, al, dims, success);
+        if (anyAbsent) {
+          ka = env.constants.absent;
+        }
       }
-      ka = eval_arrayaccess(env, al, dims, success);
     }
-    if (!success && env.inMaybePartial == 0) {
-      ResultUndefinedError warning(env, al->loc(), "array access out of bounds");
+    if (!success() && env.inMaybePartial == 0) {
+      ResultUndefinedError warning(env, Expression::loc(al), success.errorMessage(env, al));
     }
-    ees.emplace_back(nullptr, constants().boollit(success));
+    ees.emplace_back(nullptr, env.constants.boollit(success()));
     if (aa->type().isbool() && !aa->type().isOpt()) {
-      ret.b = bind(env, Ctx(), b, constants().literalTrue);
+      ret.b = bind(env, Ctx(), b, env.constants.literalTrue);
       ees.emplace_back(nullptr, ka());
       ret.r = conj(env, r, ctx, ees);
     } else {
       ret.b = conj(env, b, ctx, ees);
       ret.r = bind(env, ctx, r, ka());
     }
+  } else if (aa->type().structBT()) {
+    // x[i], where x is an array of tuples, and i is an index variable
+    // Strategy: create/flatten a seperate array access for each field, combine to new tuple literal
+    assert(Expression::type(eev.r()).bt() == aa->type().bt());
+
+    std::vector<Expression*> idx(aa->idx().size());
+    for (size_t i = 0; i < aa->idx().size(); ++i) {
+      idx[i] = ees[i].r();
+    }
+
+    StructType* res_st = env.getStructType(aa->type());
+
+    // Construct field based array access expressions
+    std::vector<KeepAlive> field_aa(res_st->size());
+    {
+      GCLock lock;
+      std::vector<Expression*> field_al = field_slices(env, eev.r());
+      assert(res_st->size() == field_al.size());
+      for (int i = 0; i < res_st->size(); ++i) {
+        field_aa[i] = new ArrayAccess(Expression::loc(aa).introduce(), field_al[i], idx);
+        Expression::type(field_aa[i](), (*res_st)[i]);
+      }
+    }
+    // Flatten field based array access expressions
+    // WARNING: Expressions stored in field_res, rely on being also in ees to be kept alive
+    std::vector<Expression*> field_res(res_st->size());
+    for (int i = 0; i < res_st->size(); ++i) {
+      // TODO: Does the context need to be changed? Are 'r' and 'b' correct?
+      EE ee = flat_exp(env, ctx, field_aa[i](), nullptr, b);
+      field_res[i] = ee.r();
+      ees.push_back(ee);
+    }
+    {
+      GCLock lock;
+      ArrayLit* tuple_lit = ArrayLit::constructTuple(Location().introduce(), field_res);
+      tuple_lit->type(aa->type());
+      ret.r = bind(env, ctx, r, tuple_lit);
+      ret.b = conj(env, b, ctx, ees);
+    }
   } else {
     std::vector<Expression*> args(aa->idx().size() + 1);
     for (unsigned int i = aa->idx().size(); (i--) != 0U;) {
@@ -263,26 +340,34 @@ flatten_arrayaccess:
     KeepAlive ka;
     {
       GCLock lock;
-      Call* cc = new Call(e->loc().introduce(), constants().ids.element, args);
+      Call* cc = Call::a(Expression::loc(e).introduce(), env.constants.ids.element, args);
       cc->type(aa->type());
-      FunctionI* fi = env.model->matchFn(env, cc->id(), args, false);
+      FunctionI* fi = nullptr;
+      try {
+        fi = env.model->matchFn(env, cc->id(), args, false);
+      } catch (TypeError&) {
+        // Actual array is bottom, but previously had a type so use that version
+        args[aa->idx().size()] = aa->v();
+        fi = env.model->matchFn(env, cc->id(), args, false);
+        args[aa->idx().size()] = eev.r();
+      }
       if (fi == nullptr) {
-        throw FlatteningError(env, cc->loc(), "cannot find matching declaration");
+        throw FlatteningError(env, Expression::loc(cc), "cannot find matching declaration");
       }
       assert(fi);
-      assert(env.isSubtype(fi->rtype(env, args, false), cc->type(), false));
+      assert(env.isSubtype(fi->rtype(env, args, nullptr, false), cc->type(), false));
       cc->decl(fi);
       ka = cc;
     }
     Ctx elemctx = ctx;
     elemctx.neg = false;
-    EE ee = flat_exp(env, elemctx, ka(), nullptr, nullptr);
+    EE ee = flat_exp(env, elemctx, ka(), nullptr, elemctx.partialityVar(env));
     ees.push_back(ee);
     if (aa->type().isbool() && !aa->type().isOpt()) {
       ee.b = ee.r;
       ees.push_back(ee);
       ret.r = conj(env, r, ctx, ees);
-      ret.b = bind(env, ctx, b, constants().boollit(!ctx.neg));
+      ret.b = bind(env, ctx, b, env.constants.boollit(!ctx.neg));
     } else {
       ret.r = bind(env, ctx, r, ee.r());
       ret.b = conj(env, b, ctx, ees);
