@@ -9,6 +9,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include <minizinc/eval_par.hh>
 #include <minizinc/flat_exp.hh>
 
 namespace MiniZinc {
@@ -17,22 +18,22 @@ EE flatten_id(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b,
               bool doNotFollowChains) {
   CallStackItem _csi(env, e);
   EE ret;
-  Id* id = e->cast<Id>();
+  Id* id = Expression::cast<Id>(e);
   if (id->decl() == nullptr) {
     if (id->type().isAnn()) {
-      ret.b = bind(env, Ctx(), b, constants().literalTrue);
+      ret.b = bind(env, Ctx(), b, env.constants.literalTrue);
       ret.r = bind(env, ctx, r, e);
       return ret;
     }
-    throw FlatteningError(env, e->loc(), "undefined identifier");
+    throw FlatteningError(env, Expression::loc(e), "undefined identifier");
   }
   if (!doNotFollowChains) {
     Expression* id_f = follow_id_to_decl(id);
-    if (id_f == constants().absent) {
-      ret.b = bind(env, Ctx(), b, constants().literalTrue);
+    if (id_f == env.constants.absent) {
+      ret.b = bind(env, Ctx(), b, env.constants.literalTrue);
       ret.r = bind(env, ctx, r, id_f);
     } else {
-      id = id_f->cast<VarDecl>()->id();
+      id = Expression::cast<VarDecl>(id_f)->id();
     }
   }
   if (ctx.neg && id->type().dim() > 0) {
@@ -43,24 +44,25 @@ EE flatten_id(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b,
     {
       GCLock lock;
       std::vector<VarDecl*> gen_id(1);
-      gen_id[0] = new VarDecl(id->loc(), new TypeInst(id->loc(), Type::parint()), env.genId(),
-                              IntLit::a(0));
+      gen_id[0] =
+          new VarDecl(Expression::loc(id), new TypeInst(Expression::loc(id), Type::parint()),
+                      env.genId(), IntLit::a(0));
 
       /// TODO: support arbitrary dimensions
       std::vector<Expression*> idxsetargs(1);
       idxsetargs[0] = id;
-      Call* idxset = new Call(id->loc().introduce(), "index_set", idxsetargs);
+      Call* idxset = Call::a(Expression::loc(id).introduce(), "index_set", idxsetargs);
       idxset->decl(env.model->matchFn(env, idxset, false));
-      idxset->type(idxset->decl()->rtype(env, idxsetargs, false));
+      idxset->type(idxset->decl()->rtype(env, idxsetargs, nullptr, false));
       Generator gen(gen_id, idxset, nullptr);
       std::vector<Expression*> idx(1);
       Generators gens;
       gens.g.push_back(gen);
-      UnOp* aanot = new UnOp(id->loc(), UOT_NOT, nullptr);
-      auto* cp = new Comprehension(id->loc(), aanot, gens, false);
+      UnOp* aanot = new UnOp(Expression::loc(id), UOT_NOT, nullptr);
+      auto* cp = new Comprehension(Expression::loc(id), aanot, gens, false);
       Id* bodyidx = cp->decl(0, 0)->id();
       idx[0] = bodyidx;
-      auto* aa = new ArrayAccess(id->loc(), id, idx);
+      auto* aa = new ArrayAccess(Expression::loc(id), id, idx);
       aanot->e(aa);
       Type tt = id->type();
       tt.dim(0);
@@ -77,95 +79,113 @@ EE flatten_id(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b,
     VarDecl* vd = id->decl()->flat();
     Expression* rete = nullptr;
     if (vd == nullptr) {
-      if (id->decl()->e() == nullptr || id->decl()->e()->type().isAnn() ||
-          id->decl()->e()->type().isvar() || id->decl()->e()->type().cv() ||
-          id->decl()->e()->type().dim() > 0) {
+      if (id->decl()->e() == nullptr || Expression::type(id->decl()->e()).isAnn() ||
+          Expression::type(id->decl()->e()).isvar() || Expression::type(id->decl()->e()).cv() ||
+          Expression::type(id->decl()->e()).dim() > 0) {
         // New top-level id, need to copy into env.m
-        vd = flat_exp(env, Ctx(), id->decl(), nullptr, constants().varTrue).r()->cast<Id>()->decl();
+        Ctx nctx;
+        nctx.i = ctx.i;
+        auto* flat_ident = Expression::cast<Id>(
+            flat_exp(env, nctx, id->decl(), nullptr, env.constants.varTrue).r());
+        if (flat_ident->decl() == nullptr && id->type().isAnn()) {
+          ret.b = bind(env, Ctx(), b, env.constants.literalTrue);
+          ret.r = bind(env, ctx, r, flat_ident);
+          return ret;
+        }
+        vd = flat_ident->decl();
       } else {
         vd = id->decl();
       }
     }
-    ret.b = bind(env, Ctx(), b, constants().literalTrue);
+    ret.b = bind(env, Ctx(), b, env.constants.literalTrue);
     if (vd->e() != nullptr) {
-      if (vd->e()->type().isPar() && vd->e()->type().dim() == 0) {
+      if (Expression::type(vd->e()).isPar() && Expression::type(vd->e()).dim() == 0) {
         rete = eval_par(env, vd->e());
         if (vd->toplevel() && (vd->ti()->domain() != nullptr) && !vd->ti()->computedDomain()) {
-          // need to check if domain includes RHS value
+          check_par_domain(env, vd, rete);
           if (vd->type() == Type::varbool()) {
-            if (!Expression::equal(rete, vd->ti()->domain())) {
-              env.fail();
-            }
             vd->ti()->domain(rete);
           } else if (vd->type() == Type::varint()) {
-            IntSetVal* isv = eval_intset(env, vd->ti()->domain());
             IntVal v = eval_int(env, rete);
-            if (!isv->contains(v)) {
-              env.fail();
-            }
             vd->ti()->domain(new SetLit(Location().introduce(), IntSetVal::a(v, v)));
           } else if (vd->type() == Type::varfloat()) {
-            FloatSetVal* fsv = eval_floatset(env, vd->ti()->domain());
             FloatVal v = eval_float(env, rete);
-            if (!fsv->contains(v)) {
-              env.fail();
-            }
             vd->ti()->domain(new SetLit(Location().introduce(), FloatSetVal::a(v, v)));
           } else if (vd->type() == Type::varsetint()) {
-            IntSetVal* isv = eval_intset(env, vd->ti()->domain());
             IntSetVal* v = eval_intset(env, rete);
-            IntSetRanges isv_r(isv);
-            IntSetRanges v_r(v);
-            if (!Ranges::subset(v_r, isv_r)) {
-              env.fail();
-            }
             vd->ti()->domain(new SetLit(Location().introduce(), v));
           }
-          // If we made it to here, the new domain is equal to the RHS
           vd->ti()->setComputedDomain(true);
         }
-      } else if (vd->e()->isa<Id>()) {
+      } else if (Expression::isa<Id>(vd->e())) {
         rete = vd->e();
       }
-    } else if (vd->ti()->ranges().size() == 0 && (vd->ti()->domain() != nullptr) &&
+    } else if (vd->ti()->ranges().empty() && (vd->ti()->domain() != nullptr) &&
                vd->type().st() == Type::ST_PLAIN && vd->type().ot() == Type::OT_PRESENT) {
       if (vd->type().bt() == Type::BT_BOOL) {
         rete = vd->ti()->domain();
-      } else if (vd->type().bt() == Type::BT_INT && vd->ti()->domain()->isa<SetLit>() &&
-                 (vd->ti()->domain()->cast<SetLit>()->isv() != nullptr) &&
-                 vd->ti()->domain()->cast<SetLit>()->isv()->card() == 1) {
-        rete = IntLit::a(vd->ti()->domain()->cast<SetLit>()->isv()->min());
+      } else if (Expression::type(vd).bt() == Type::BT_INT &&
+                 Expression::isa<SetLit>(vd->ti()->domain()) &&
+                 (Expression::cast<SetLit>(vd->ti()->domain())->isv() != nullptr) &&
+                 Expression::cast<SetLit>(vd->ti()->domain())->isv()->card() == 1) {
+        rete = IntLit::a(Expression::cast<SetLit>(vd->ti()->domain())->isv()->min());
+      } else if (vd->ti()->type().structBT()) {
+        auto* fieldsti = Expression::cast<ArrayLit>(vd->ti()->domain());
+        std::vector<Expression*> elems(fieldsti->size());
+        for (size_t i = 0; i < fieldsti->size(); ++i) {
+          auto* nti = Expression::cast<TypeInst>((*fieldsti)[i]);
+          Type nty(nti->type());
+          auto* vti = new TypeInst(Location().introduce(), nty, nti->ranges(), nti->domain());
+          VarDecl* nvd = new_vardecl(env, Ctx(), vti, nullptr, vd, nullptr);
+          elems[i] =
+              flatten_id(env, ctx, nvd->id(), nullptr, env.constants.varTrue, doNotFollowChains)
+                  .r();
+        }
+        // After introducing variables for each tuple element, the original domain can be
+        // set to "computed" (since it is a consequence of the individual variable domains)
+        vd->ti()->setComputedDomain(true);
+
+        ArrayLit* al = ArrayLit::constructTuple(Location().introduce(), elems);
+        al->type(vd->type());
+
+        // Set tuple instantiation as RHS
+        vd->e(al);
+
+        // count flattened version and add to CSE
+        env.voAddExp(vd);
+        EE ee;
+        ee.r = vd;
+        env.cseMapInsert(vd->e(), ee);
       }
-    } else if (vd->ti()->ranges().size() > 0) {
+    } else if (!vd->ti()->ranges().empty()) {
       // create fresh variables and array literal
       std::vector<std::pair<int, int> > dims;
       IntVal asize = 1;
       for (unsigned int i = 0; i < vd->ti()->ranges().size(); i++) {
         TypeInst* ti = vd->ti()->ranges()[i];
         if (ti->domain() == nullptr) {
-          throw FlatteningError(env, ti->loc(), "array dimensions unknown");
+          throw FlatteningError(env, Expression::loc(ti), "array dimensions unknown");
         }
         IntSetVal* isv = eval_intset(env, ti->domain());
-        if (isv->size() == 0) {
+        if (isv->empty()) {
           dims.emplace_back(1, 0);
           asize = 0;
         } else {
           if (isv->size() != 1) {
-            throw FlatteningError(env, ti->loc(), "invalid array index set");
+            throw FlatteningError(env, Expression::loc(ti), "invalid array index set");
           }
           asize *= (isv->max(0) - isv->min(0) + 1);
           dims.emplace_back(static_cast<int>(isv->min(0).toInt()),
                             static_cast<int>(isv->max(0).toInt()));
         }
       }
-      Type tt = vd->ti()->type();
-      tt.dim(0);
+      Type tt = vd->ti()->type().elemType(env);
 
       if (asize > Constants::max_array_size) {
         std::ostringstream oss;
         oss << "array size (" << asize << ") exceeds maximum allowed size ("
             << Constants::max_array_size << ")";
-        throw FlatteningError(env, vd->loc(), oss.str());
+        throw FlatteningError(env, Expression::loc(vd), oss.str());
       }
 
       std::vector<Expression*> elems(static_cast<int>(asize.toInt()));
@@ -174,13 +194,18 @@ EE flatten_id(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b,
         auto* vti = new TypeInst(Location().introduce(), tt, vd->ti()->domain());
         VarDecl* nvd = new_vardecl(env, Ctx(), vti, nullptr, vd, nullptr);
         elems[i] = nvd->id();
+        if (tt.structBT()) {
+          elems[i] =
+              flatten_id(env, ctx, nvd->id(), nullptr, env.constants.varTrue, doNotFollowChains)
+                  .r();
+        }
       }
       // After introducing variables for each array element, the original domain can be
       // set to "computed" (since it is a consequence of the individual variable domains)
       vd->ti()->setComputedDomain(true);
 
       auto* al = new ArrayLit(Location().introduce(), elems, dims);
-      al->type(vd->type());
+      al->type(elems.empty() ? Type::bot(vd->type().dim()) : vd->type());
       vd->e(al);
       env.voAddExp(vd);
       EE ee;
@@ -193,15 +218,15 @@ EE flatten_id(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b,
         auto it = env.cseMapFind(vd->e());
         if (it == env.cseMapEnd()) {
           Expression* vde = follow_id(vd->e());
-          ArrayLit* vdea = vde != nullptr ? vde->dynamicCast<ArrayLit>() : nullptr;
-          if ((vdea != nullptr) && vdea->size() == 0) {
+          auto* vdea = Expression::dynamicCast<ArrayLit>(vde);
+          if ((vdea != nullptr) && vdea->empty()) {
             // Do not create names for empty arrays but return array literal directly
             rete = vdea;
           } else {
             VarDecl* nvd = new_vardecl(env, ctx, eval_typeinst(env, ctx, vd), nullptr, vd, nullptr);
 
             if (vd->e() != nullptr) {
-              (void)flat_exp(env, Ctx(), vd->e(), nvd, constants().varTrue);
+              (void)flat_exp(env, Ctx(), vd->e(), nvd, env.constants.varTrue);
             }
             vd = nvd;
             EE ee(vd, nullptr);
@@ -210,10 +235,10 @@ EE flatten_id(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b,
             }
           }
         } else {
-          if (it->second.r()->isa<VarDecl>()) {
-            vd = it->second.r()->cast<VarDecl>();
+          if (Expression::isa<VarDecl>(it->second.r)) {
+            vd = Expression::cast<VarDecl>(it->second.r);
           } else {
-            rete = it->second.r();
+            rete = it->second.r;
           }
         }
       }
@@ -221,8 +246,8 @@ EE flatten_id(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b,
         if (id->type().bt() == Type::BT_ANN && (vd->e() != nullptr)) {
           rete = vd->e();
         } else {
-          auto* vda = vd->dynamicCast<ArrayLit>();
-          if ((vda != nullptr) && vda->size() == 0) {
+          auto* vda = Expression::dynamicCast<ArrayLit>(vd);
+          if ((vda != nullptr) && vda->empty()) {
             // Do not create names for empty arrays but return array literal directly
             rete = vda;
           } else {
@@ -231,6 +256,13 @@ EE flatten_id(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b,
         }
       }
     }
+    // Add reverse mapper for tuple var decls
+    // TODO: This only has to happen on first flatten_id call.
+    if ((vd->type().istuple() || vd->type().isrecord()) && vd->e() != nullptr) {
+      Expression* lit = follow_id(vd->e());
+      assert(Expression::isa<ArrayLit>(lit));
+      env.reverseMappers.insert(vd->id(), lit);
+    }
     ret.r = bind(env, ctx, r, rete);
   }
   return ret;
