@@ -9,7 +9,12 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include <minizinc/ast.hh>
 #include <minizinc/flat_exp.hh>
+#include <minizinc/flatten_internal.hh>
+#include <minizinc/gc.hh>
+
+#include <vector>
 
 namespace MiniZinc {
 
@@ -20,7 +25,7 @@ std::vector<Expression*> get_conjuncts(Expression* start) {
   while (!conj_stack.empty()) {
     Expression* e = conj_stack.back();
     conj_stack.pop_back();
-    if (auto* bo = e->dynamicCast<BinOp>()) {
+    if (auto* bo = Expression::dynamicCast<BinOp>(e)) {
       if (bo->op() == BOT_AND) {
         conj_stack.push_back(bo->rhs());
         conj_stack.push_back(bo->lhs());
@@ -34,42 +39,95 @@ std::vector<Expression*> get_conjuncts(Expression* start) {
   return conjuncts;
 }
 
-void classify_conjunct(Expression* e, IdMap<int>& eq_occurrences,
+void classify_conjunct(EnvI& env, Expression* e, IdMap<int>& eq_occurrences,
                        IdMap<std::pair<Expression*, Expression*>>& eq_branches,
                        std::vector<Expression*>& other_branches) {
-  if (auto* bo = e->dynamicCast<BinOp>()) {
-    if (bo->op() == BOT_EQ) {
-      if (Id* ident = bo->lhs()->dynamicCast<Id>()) {
-        if (eq_branches.find(ident) == eq_branches.end()) {
-          auto it = eq_occurrences.find(ident);
-          if (it == eq_occurrences.end()) {
-            eq_occurrences.insert(ident, 1);
-          } else {
-            eq_occurrences.get(ident)++;
-          }
-          eq_branches.insert(ident, std::make_pair(bo->rhs(), bo));
-          return;
-        }
-      } else if (Id* ident = bo->rhs()->dynamicCast<Id>()) {
-        if (eq_branches.find(ident) == eq_branches.end()) {
-          auto it = eq_occurrences.find(ident);
-          if (it == eq_occurrences.end()) {
-            eq_occurrences.insert(ident, 1);
-          } else {
-            eq_occurrences.get(ident)++;
-          }
-          eq_branches.insert(ident, std::make_pair(bo->lhs(), bo));
-          return;
+  if (auto* bo = Expression::dynamicCast<BinOp>(e)) {
+    if (bo->op() == BOT_EQ && Expression::type(bo->lhs()).dim() == 0) {
+      auto* ident = Expression::dynamicCast<Id>(bo->lhs());
+      auto* other = bo->rhs();
+      if (ident == nullptr || ident == env.constants.absent) {
+        ident = Expression::dynamicCast<Id>(bo->rhs());
+        other = bo->lhs();
+      }
+      if (ident != nullptr && ident != env.constants.absent &&
+          eq_branches.find(ident) == eq_branches.end()) {
+        auto it = eq_occurrences.find(ident);
+        if (it == eq_occurrences.end()) {
+          eq_occurrences.insert(ident, 1);
+        } else {
+          eq_occurrences.get(ident)++;
         }
+        eq_branches.insert(ident, std::make_pair(other, bo));
+        return;
       }
     }
   }
   other_branches.push_back(e);
 }
 
+Expression* ite_struct_split(EnvI& env, Type ty, const std::vector<Expression*>& then_in,
+                             Expression* else_in, std::vector<KeepAlive>& results,
+                             std::vector<std::vector<KeepAlive>>& e_then,
+                             std::vector<KeepAlive>& e_else) {
+  StructType* st = env.getStructType(ty);
+  GCLock lock;
+  std::vector<Expression*> tupleResult(st->size());
+
+  std::vector<VarDecl*> then_decl(then_in.size());
+  for (int i = 0; i < then_in.size(); ++i) {
+    then_decl[i] =
+        new VarDecl(Location().introduce(),
+                    new TypeInst(Location().introduce(), Expression::type(then_in[i]), nullptr),
+                    env.genId(), then_in[i]);
+    then_decl[i]->ti()->setStructDomain(env, Expression::type(then_in[i]));
+  }
+  auto* else_decl =
+      new VarDecl(Location().introduce(),
+                  new TypeInst(Location().introduce(), Expression::type(else_in), nullptr),
+                  env.genId(), else_in);
+  else_decl->ti()->setStructDomain(env, Expression::type(else_in));
+
+  for (int i = 0; i < st->size(); ++i) {
+    Type field = (*st)[i];
+    if (field.structBT()) {
+      std::vector<Expression*> f_then_in(then_decl.size());
+      for (int j = 0; j < then_decl.size(); ++j) {
+        f_then_in[j] =
+            new FieldAccess(Location().introduce(), then_decl[i]->id(), IntLit::a(i + 1));
+        Expression::type(f_then_in[j], field);
+      }
+      Expression* f_else_in =
+          new FieldAccess(Location().introduce(), else_decl->id(), IntLit::a(i + 1));
+      tupleResult[i] = ite_struct_split(env, field, f_then_in, f_else_in, results, e_then, e_else);
+    } else {
+      VarDecl* fieldRes =
+          new_vardecl(env, Ctx(), new TypeInst(Location().introduce(), field, nullptr), nullptr,
+                      nullptr, nullptr);
+      tupleResult[i] = fieldRes->id();
+
+      results.emplace_back(fieldRes);
+      e_then.emplace_back();
+      e_then.back().reserve(then_decl.size());
+      for (auto& decl : then_decl) {
+        e_then.back().push_back(
+            new FieldAccess(Location().introduce(), decl->id(), IntLit::a(i + 1)));
+        Expression::type(e_then.back().back()(), field);
+      }
+      e_else.emplace_back(
+          new FieldAccess(Location().introduce(), else_decl->id(), IntLit::a(i + 1)));
+      Expression::type(e_else.back()(), field);
+    }
+    assert(results.size() == e_then.size() && results.size() == e_else.size());
+  }
+  ArrayLit* tuple = ArrayLit::constructTuple(Location().introduce(), tupleResult);
+  tuple->type(ty);
+  return tuple;
+}
+
 EE flatten_ite(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b) {
   CallStackItem _csi(env, e);
-  ITE* ite = e->cast<ITE>();
+  ITE* ite = Expression::cast<ITE>(e);
 
   // The conditions of each branch of the if-then-else
   std::vector<KeepAlive> conditions;
@@ -92,14 +150,15 @@ EE flatten_ite(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b)
   bool allDefined = true;
 
   // The result variables of each generated conditional
-  std::vector<VarDecl*> results;
+  std::vector<KeepAlive> results;
+  KeepAlive tupleResult;
   // The then-expressions of each generated conditional
   std::vector<std::vector<KeepAlive>> e_then;
   // The else-expressions of each generated conditional
   std::vector<KeepAlive> e_else;
 
   bool noOtherBranches = true;
-  if (ite->type() == Type::varbool() && ctx.b == C_ROOT && r == constants().varTrue) {
+  if (ite->type() == Type::varbool() && ctx.b == C_ROOT && r == env.constants.varTrue) {
     // Check if all branches are of the form x1=e1 /\ ... /\ xn=en
     IdMap<int> eq_occurrences;
     std::vector<IdMap<std::pair<Expression*, Expression*>>> eq_branches(ite->size() + 1);
@@ -107,29 +166,30 @@ EE flatten_ite(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b)
     for (int i = 0; i < ite->size(); i++) {
       auto conjuncts = get_conjuncts(ite->thenExpr(i));
       for (auto* c : conjuncts) {
-        classify_conjunct(c, eq_occurrences, eq_branches[i], other_branches[i]);
+        classify_conjunct(env, c, eq_occurrences, eq_branches[i], other_branches[i]);
       }
       noOtherBranches = noOtherBranches && other_branches[i].empty();
     }
     {
       auto conjuncts = get_conjuncts(ite->elseExpr());
       for (auto* c : conjuncts) {
-        classify_conjunct(c, eq_occurrences, eq_branches[ite->size()], other_branches[ite->size()]);
+        classify_conjunct(env, c, eq_occurrences, eq_branches[ite->size()],
+                          other_branches[ite->size()]);
       }
       noOtherBranches = noOtherBranches && other_branches[ite->size()].empty();
     }
     for (auto& eq : eq_occurrences) {
       if (eq.second >= ite->size()) {
         // Any identifier that occurs in all or all but one branch gets its own conditional
-        results.push_back(eq.first->decl());
+        results.emplace_back(eq.first->decl());
         e_then.emplace_back();
         for (int i = 0; i < ite->size(); i++) {
           auto it = eq_branches[i].find(eq.first);
           if (it == eq_branches[i].end()) {
             // not found, simply push x=x
-            e_then.back().push_back(eq.first);
+            e_then.back().emplace_back(eq.first);
           } else {
-            e_then.back().push_back(it->second.first);
+            e_then.back().emplace_back(it->second.first);
           }
         }
         {
@@ -154,49 +214,57 @@ EE flatten_ite(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b)
       }
     }
     if (!noOtherBranches) {
-      results.push_back(r);
+      results.emplace_back(r);
       e_then.emplace_back();
       for (int i = 0; i < ite->size(); i++) {
-        if (eq_branches[i].size() == 0) {
-          e_then.back().push_back(ite->thenExpr(i));
+        if (eq_branches[i].empty()) {
+          e_then.back().emplace_back(ite->thenExpr(i));
         } else if (other_branches[i].empty()) {
-          e_then.back().push_back(constants().literalTrue);
+          e_then.back().emplace_back(env.constants.literalTrue);
         } else if (other_branches[i].size() == 1) {
-          e_then.back().push_back(other_branches[i][0]);
+          e_then.back().emplace_back(other_branches[i][0]);
         } else {
           GCLock lock;
           auto* al = new ArrayLit(Location().introduce(), other_branches[i]);
           al->type(Type::varbool(1));
-          Call* forall = new Call(Location().introduce(), constants().ids.forall, {al});
+          Call* forall = Call::a(Location().introduce(), env.constants.ids.forall, {al});
           forall->decl(env.model->matchFn(env, forall, false));
-          forall->type(forall->decl()->rtype(env, {al}, false));
-          e_then.back().push_back(forall);
+          forall->type(forall->decl()->rtype(env, {al}, nullptr, false));
+          e_then.back().emplace_back(forall);
         }
       }
       {
-        if (eq_branches[ite->size()].size() == 0) {
+        if (eq_branches[ite->size()].empty()) {
           e_else.emplace_back(ite->elseExpr());
         } else if (other_branches[ite->size()].empty()) {
-          e_else.emplace_back(constants().literalTrue);
+          e_else.emplace_back(env.constants.literalTrue);
         } else if (other_branches[ite->size()].size() == 1) {
           e_else.emplace_back(other_branches[ite->size()][0]);
         } else {
           GCLock lock;
           auto* al = new ArrayLit(Location().introduce(), other_branches[ite->size()]);
           al->type(Type::varbool(1));
-          Call* forall = new Call(Location().introduce(), constants().ids.forall, {al});
+          Call* forall = Call::a(Location().introduce(), env.constants.ids.forall, {al});
           forall->decl(env.model->matchFn(env, forall, false));
-          forall->type(forall->decl()->rtype(env, {al}, false));
+          forall->type(forall->decl()->rtype(env, {al}, nullptr, false));
           e_else.emplace_back(forall);
         }
       }
     }
+  } else if (ite->type().istuple() || ite->type().isrecord()) {
+    noOtherBranches = false;
+    std::vector<Expression*> then_in(ite->size());
+    for (size_t i = 0; i < ite->size(); ++i) {
+      then_in[i] = ite->thenExpr(i);
+    }
+    tupleResult =
+        ite_struct_split(env, ite->type(), then_in, ite->elseExpr(), results, e_then, e_else);
   } else {
     noOtherBranches = false;
-    results.push_back(r);
+    results.emplace_back(r);
     e_then.emplace_back();
     for (int i = 0; i < ite->size(); i++) {
-      e_then.back().push_back(ite->thenExpr(i));
+      e_then.back().emplace_back(ite->thenExpr(i));
     }
     e_else.emplace_back(ite->elseExpr());
   }
@@ -225,16 +293,16 @@ EE flatten_ite(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b)
   for (int i = 0; i < ite->size() && !foundTrueBranch; i++) {
     bool cond = true;
     EE e_if;
-    if (ite->ifExpr(i)->isa<Call>() &&
-        ite->ifExpr(i)->cast<Call>()->id() == "mzn_in_root_context") {
-      e_if = EE(constants().boollit(ctx.b == C_ROOT), constants().literalTrue);
+    if (Expression::isa<Call>(ite->ifExpr(i)) &&
+        Expression::cast<Call>(ite->ifExpr(i))->id() == env.constants.ids.mzn_in_root_context) {
+      e_if = EE(env.constants.boollit(ctx.b == C_ROOT), env.constants.literalTrue);
     } else {
       Ctx cmix_not_negated;
       cmix_not_negated.b = C_MIX;
       cmix_not_negated.i = C_MIX;
-      e_if = flat_exp(env, cmix_not_negated, ite->ifExpr(i), nullptr, constants().varTrue);
+      e_if = flat_exp(env, cmix_not_negated, ite->ifExpr(i), nullptr, env.constants.varTrue);
     }
-    if (e_if.r()->type() == Type::parbool()) {
+    if (Expression::type(e_if.r()) == Type::parbool()) {
       {
         GCLock lock;
         cond = eval_bool(env, e_if.r());
@@ -248,24 +316,24 @@ EE flatten_ite(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b)
         // had var conditions, so we have to take them into account
         // and emit new conditional clause
         // add another condition and definedness variable
-        conditions.emplace_back(constants().literalTrue);
+        conditions.emplace_back(env.constants.literalTrue);
         for (unsigned int j = 0; j < results.size(); j++) {
-          EE ethen = flat_exp(env, cmix, e_then[j][i](), nullptr, nullptr);
+          EE ethen = flat_exp(env, cmix, e_then[j][i](), nullptr, cmix.partialityVar(env));
           assert(ethen.b());
           defined[j].push_back(ethen.b);
-          allDefined = allDefined && (ethen.b() == constants().literalTrue);
+          allDefined = allDefined && (ethen.b() == env.constants.literalTrue);
           branches[j].push_back(ethen.r);
-          if (ethen.r()->type().isvar()) {
+          if (Expression::type(ethen.r()).isvar()) {
             allBranchesPar[j] = false;
           }
         }
         foundTrueBranch = true;
       } else {
         GCLock lock;
-        conditions.emplace_back(constants().literalFalse);
+        conditions.emplace_back(env.constants.literalFalse);
         for (unsigned int j = 0; j < results.size(); j++) {
-          defined[j].push_back(constants().literalTrue);
-          branches[j].push_back(create_dummy_value(env, e_then[j][i]()->type()));
+          defined[j].emplace_back(env.constants.literalTrue);
+          branches[j].emplace_back(create_dummy_value(env, Expression::type(e_then[j][i]())));
         }
       }
     } else {
@@ -275,13 +343,13 @@ EE flatten_ite(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b)
 
       for (unsigned int j = 0; j < results.size(); j++) {
         // flatten the then branch
-        EE ethen = flat_exp(env, cmix, e_then[j][i](), nullptr, nullptr);
+        EE ethen = flat_exp(env, cmix, e_then[j][i](), nullptr, cmix.partialityVar(env));
 
         assert(ethen.b());
         defined[j].push_back(ethen.b);
-        allDefined = allDefined && (ethen.b() == constants().literalTrue);
+        allDefined = allDefined && (ethen.b() == env.constants.literalTrue);
         branches[j].push_back(ethen.r);
-        if (ethen.r()->type().isvar()) {
+        if (Expression::type(ethen.r()).isvar()) {
           allBranchesPar[j] = false;
         }
       }
@@ -290,21 +358,21 @@ EE flatten_ite(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b)
 
     if (cond) {
       for (unsigned int j = 0; j < results.size(); j++) {
-        if (r_bounds_valid_int[j] && e_then[j][i]()->type().isint()) {
+        if (r_bounds_valid_int[j] && Expression::type(e_then[j][i]()).isint()) {
           GCLock lock;
           IntBounds ib_then = compute_int_bounds(env, branches[j][i]());
           if (ib_then.valid) {
             r_bounds_int[j].push_back(ib_then);
           }
           r_bounds_valid_int[j] = r_bounds_valid_int[j] && ib_then.valid;
-        } else if (r_bounds_valid_set[j] && e_then[j][i]()->type().isIntSet()) {
+        } else if (r_bounds_valid_set[j] && Expression::type(e_then[j][i]()).isIntSet()) {
           GCLock lock;
           IntSetVal* isv = compute_intset_bounds(env, branches[j][i]());
           if (isv != nullptr) {
             r_bounds_set[j].push_back(isv);
           }
           r_bounds_valid_set[j] = r_bounds_valid_set[j] && (isv != nullptr);
-        } else if (r_bounds_valid_float[j] && e_then[j][i]()->type().isfloat()) {
+        } else if (r_bounds_valid_float[j] && Expression::type(e_then[j][i]()).isfloat()) {
           GCLock lock;
           FloatBounds fb_then = compute_float_bounds(env, branches[j][i]());
           if (fb_then.valid) {
@@ -323,7 +391,7 @@ EE flatten_ite(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b)
   }
 
   for (auto& result : results) {
-    if (result == nullptr) {
+    if (result() == nullptr) {
       // need to introduce new result variable
       GCLock lock;
       auto* ti = new TypeInst(Location().introduce(), ite->type(), nullptr);
@@ -331,36 +399,36 @@ EE flatten_ite(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b)
     }
   }
 
-  if (conditions.back()() != constants().literalTrue) {
+  if (conditions.back()() != env.constants.literalTrue) {
     // The last condition wasn't fixed to true, we need to look at the else branch
-    conditions.emplace_back(constants().literalTrue);
+    conditions.emplace_back(env.constants.literalTrue);
 
     for (unsigned int j = 0; j < results.size(); j++) {
       // flatten else branch
-      EE eelse = flat_exp(env, cmix, e_else[j](), nullptr, nullptr);
+      EE eelse = flat_exp(env, cmix, e_else[j](), nullptr, cmix.partialityVar(env));
       assert(eelse.b());
       defined[j].push_back(eelse.b);
-      allDefined = allDefined && (eelse.b() == constants().literalTrue);
+      allDefined = allDefined && (eelse.b() == env.constants.literalTrue);
       branches[j].push_back(eelse.r);
-      if (eelse.r()->type().isvar()) {
+      if (Expression::type(eelse.r()).isvar()) {
         allBranchesPar[j] = false;
       }
 
-      if (r_bounds_valid_int[j] && e_else[j]()->type().isint()) {
+      if (r_bounds_valid_int[j] && Expression::type(e_else[j]()).isint()) {
         GCLock lock;
         IntBounds ib_else = compute_int_bounds(env, eelse.r());
         if (ib_else.valid) {
           r_bounds_int[j].push_back(ib_else);
         }
         r_bounds_valid_int[j] = r_bounds_valid_int[j] && ib_else.valid;
-      } else if (r_bounds_valid_set[j] && e_else[j]()->type().isIntSet()) {
+      } else if (r_bounds_valid_set[j] && Expression::type(e_else[j]()).isIntSet()) {
         GCLock lock;
         IntSetVal* isv = compute_intset_bounds(env, eelse.r());
         if (isv != nullptr) {
           r_bounds_set[j].push_back(isv);
         }
         r_bounds_valid_set[j] = r_bounds_valid_set[j] && (isv != nullptr);
-      } else if (r_bounds_valid_float[j] && e_else[j]()->type().isfloat()) {
+      } else if (r_bounds_valid_float[j] && Expression::type(e_else[j]()).isfloat()) {
         GCLock lock;
         FloatBounds fb_else = compute_float_bounds(env, eelse.r());
         if (fb_else.valid) {
@@ -372,9 +440,8 @@ EE flatten_ite(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b)
   }
 
   // update domain of result variable with bounds from all branches
-
   for (unsigned int j = 0; j < results.size(); j++) {
-    VarDecl* nr = results[j];
+    auto* nr = Expression::cast<VarDecl>(results[j]());
     GCLock lock;
     if (r_bounds_valid_int[j] && ite->type().isint()) {
       IntVal lb = IntVal::infinity();
@@ -389,7 +456,7 @@ EE flatten_ite(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b)
         IntSetRanges isv_r(isv);
         Ranges::Inter<IntVal, Ranges::Const<IntVal>, IntSetRanges> inter(ite_r, isv_r);
         IntSetVal* isv_new = IntSetVal::ai(inter);
-        if (isv_new->card() != isv->card()) {
+        if (!isv_new->equal(isv)) {
           auto* r_dom = new SetLit(Location().introduce(), isv_new);
           nr->ti()->domain(r_dom);
         }
@@ -412,7 +479,7 @@ EE flatten_ite(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b)
         IntSetRanges isv_branches_r(isv_branches);
         Ranges::Inter<IntVal, IntSetRanges, IntSetRanges> inter(isv_branches_r, isv_r);
         IntSetVal* isv_new = IntSetVal::ai(inter);
-        if (isv_new->card() != isv->card()) {
+        if (!isv_new->equal(isv)) {
           auto* r_dom = new SetLit(Location().introduce(), isv_new);
           nr->ti()->domain(r_dom);
         }
@@ -445,68 +512,89 @@ EE flatten_ite(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b)
   }
 
   // Create ite predicate calls
-  GCLock lock;
-  auto* al_cond = new ArrayLit(Location().introduce(), conditions);
-  al_cond->type(Type::varbool(1));
+  KeepAlive al_cond;
+  {
+    GCLock lock;
+    al_cond = new ArrayLit(Location().introduce(), conditions);
+    Expression::type(al_cond(), Type::varbool(1));
+  }
   for (unsigned int j = 0; j < results.size(); j++) {
-    auto* al_branches = new ArrayLit(Location().introduce(), branches[j]);
-    Type branches_t = results[j]->type();
-    branches_t.dim(1);
-    branches_t.ti(allBranchesPar[j] ? Type::TI_PAR : Type::TI_VAR);
-    al_branches->type(branches_t);
-    Call* ite_pred = new Call(ite->loc().introduce(), ASTString("if_then_else"),
-                              {al_cond, al_branches, results[j]->id()});
-    ite_pred->decl(env.model->matchFn(env, ite_pred, false));
-    ite_pred->type(Type::varbool());
-    (void)flat_exp(env, Ctx(), ite_pred, constants().varTrue, constants().varTrue);
+    KeepAlive ite_pred;
+    {
+      GCLock lock;
+      auto* al_branches = new ArrayLit(Location().introduce(), branches[j]);
+      Type branches_t = Type::arrType(env, Type::bot(1), Expression::type(results[j]()));
+      if (!allBranchesPar[j]) {
+        branches_t.mkVar(env);
+      }
+      branches_t.ti(allBranchesPar[j] ? Type::TI_PAR : Type::TI_VAR);
+      al_branches->type(branches_t);
+      ite_pred = Call::a(Expression::loc(ite).introduce(), ASTString("if_then_else"),
+                         {al_cond(), al_branches, Expression::cast<VarDecl>(results[j]())->id()});
+      Expression::cast<Call>(ite_pred())
+          ->decl(env.model->matchFn(env, Expression::cast<Call>(ite_pred()), false));
+      Expression::type(Expression::cast<Call>(ite_pred()), Type::varbool());
+      make_defined_var(env, Expression::cast<VarDecl>(results[j]()),
+                       Expression::cast<Call>(ite_pred()));
+    }
+    (void)flat_exp(env, Ctx(), ite_pred(), env.constants.varTrue, env.constants.varTrue);
   }
   EE ret;
   if (noOtherBranches) {
-    ret.r = constants().varTrue->id();
+    ret.r = env.constants.varTrue->id();
+  } else if (ite->type().istuple() || ite->type().isrecord()) {
+    ret.r = bind(env, ctx, r, tupleResult());
   } else {
-    ret.r = results.back()->id();
+    ret.r = Expression::cast<VarDecl>(results.back()())->id();
   }
   if (allDefined) {
-    bind(env, Ctx(), b, constants().literalTrue);
-    ret.b = constants().literalTrue;
+    bind(env, Ctx(), b, env.constants.literalTrue);
+    ret.b = env.constants.literalTrue;
   } else {
     // Otherwise, constraint linking conditions, b and the definedness variables
-    if (b == nullptr) {
-      CallStackItem _csi(env, new StringLit(Location().introduce(), "b"));
-      b = new_vardecl(env, Ctx(), new TypeInst(Location().introduce(), Type::varbool()), nullptr,
-                      nullptr, nullptr);
-    }
-    ret.b = b->id();
+    KeepAlive ite_defined_pred;
+    {
+      GCLock lock;
 
-    std::vector<Expression*> defined_conjunctions(ite->size() + 1);
-    for (unsigned int i = 0; i < ite->size() + 1; i++) {
-      std::vector<Expression*> def_i;
-      for (auto& j : defined) {
-        assert(j.size() > i);
-        if (j[i]() != constants().literalTrue) {
-          def_i.push_back(j[i]());
+      if (b == nullptr) {
+        CallStackItem _csi(env, new StringLit(Location().introduce(), "b"));
+        b = new_vardecl(env, Ctx(), new TypeInst(Location().introduce(), Type::varbool()), nullptr,
+                        nullptr, nullptr);
+      }
+      ret.b = b->id();
+
+      std::vector<Expression*> defined_conjunctions(conditions.size());
+      for (unsigned int i = 0; i < conditions.size(); i++) {
+        std::vector<Expression*> def_i;
+        for (auto& j : defined) {
+          assert(j.size() > i);
+          if (j[i]() != env.constants.literalTrue) {
+            def_i.push_back(j[i]());
+          }
+        }
+        if (def_i.empty()) {
+          defined_conjunctions[i] = env.constants.literalTrue;
+        } else if (def_i.size() == 1) {
+          defined_conjunctions[i] = def_i[0];
+        } else {
+          auto* al = new ArrayLit(Location().introduce(), def_i);
+          al->type(Type::varbool(1));
+          Call* forall = Call::a(Location().introduce(), env.constants.ids.forall, {al});
+          forall->decl(env.model->matchFn(env, forall, false));
+          forall->type(forall->decl()->rtype(env, {al}, nullptr, false));
+          defined_conjunctions[i] = forall;
         }
       }
-      if (def_i.empty()) {
-        defined_conjunctions[i] = constants().literalTrue;
-      } else if (def_i.size() == 1) {
-        defined_conjunctions[i] = def_i[0];
-      } else {
-        auto* al = new ArrayLit(Location().introduce(), def_i);
-        al->type(Type::varbool(1));
-        Call* forall = new Call(Location().introduce(), constants().ids.forall, {al});
-        forall->decl(env.model->matchFn(env, forall, false));
-        forall->type(forall->decl()->rtype(env, {al}, false));
-        defined_conjunctions[i] = forall;
-      }
+      auto* al_defined = new ArrayLit(Location().introduce(), defined_conjunctions);
+      al_defined->type(Type::varbool(1));
+      ite_defined_pred =
+          Call::a(Expression::loc(ite).introduce(), ASTString("if_then_else_partiality"),
+                  {al_cond(), al_defined, b->id()});
+      Expression::cast<Call>(ite_defined_pred())
+          ->decl(env.model->matchFn(env, Expression::cast<Call>(ite_defined_pred()), false));
+      Expression::type(ite_defined_pred(), Type::varbool());
     }
-    auto* al_defined = new ArrayLit(Location().introduce(), defined_conjunctions);
-    al_defined->type(Type::varbool(1));
-    Call* ite_defined_pred = new Call(ite->loc().introduce(), ASTString("if_then_else_partiality"),
-                                      {al_cond, al_defined, b->id()});
-    ite_defined_pred->decl(env.model->matchFn(env, ite_defined_pred, false));
-    ite_defined_pred->type(Type::varbool());
-    (void)flat_exp(env, Ctx(), ite_defined_pred, constants().varTrue, constants().varTrue);
+    (void)flat_exp(env, Ctx(), ite_defined_pred(), env.constants.varTrue, env.constants.varTrue);
   }
 
   return ret;
