@@ -6,7 +6,7 @@
  */
 
 /* This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was ! distributed with this
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* A basic mzn2fzn wrapper, can be used as a plugin
@@ -16,6 +16,7 @@
 #define _CRT_SECURE_NO_WARNINGS
 #endif
 
+#include <minizinc/file_utils.hh>
 #include <minizinc/flattener.hh>
 #include <minizinc/pathfileprinter.hh>
 #include <minizinc/statistics.hh>
@@ -62,15 +63,20 @@ void Flattener::printHelp(ostream& os) const {
      << "  -m <file>, --model <file>\n    File named <file> is the model." << std::endl
      << "  -d <file>, --data <file>\n    File named <file> contains data used by the model."
      << std::endl
+     << "  --checker <file>, --solution-checker <file>\n    File named <file> contains the "
+        "solution checker model."
+     << std::endl
      << "  -D <data>, --cmdline-data <data>\n    Include the given data assignment in the model."
      << std::endl
      << "  --cmdline-json-data <data>\n    Include the given JSON data in the model." << std::endl
      << "  --stdlib-dir <dir>\n    Path to MiniZinc standard library directory" << std::endl
      << "  -G <dir>, --globals-dir <dir>, --mzn-globals-dir <dir>\n    Search for included "
         "globals "
-        "in <stdlib>/<dir>."
+        "in <stdlib>/<dir>, or <dir> when given a absolute or relative path."
+     << std::endl
+     << "  -, --input-from-stdin\n    Read problem from standard input. Combine with "
+        "--input-is-flatzinc when passing FlatZinc code."
      << std::endl
-     << "  -, --input-from-stdin\n    Read problem from standard input" << std::endl
      << "  -I <dir>, --search-dir <dir>\n    Additionally search for included files in <dir>."
      << std::endl
      << "  -D \"fMIPdomains=true\"\n    Switch on MIPDomain Unification" << std::endl
@@ -167,6 +173,8 @@ bool Flattener::processOption(int& i, std::vector<std::string>& argv,
 
   if (cop.getOption("-I --search-dir", &buffer)) {
     _includePaths.push_back(FileUtils::file_path(buffer + "/", workingDir));
+  } else if (cop.getOption("--ignore-stdlib")) {
+    _flags.ignoreStdlib = true;
   } else if (cop.getOption("--no-typecheck")) {
     _flags.typecheck = false;
   } else if (cop.getOption("--instance-check-only")) {
@@ -255,7 +263,22 @@ bool Flattener::processOption(int& i, std::vector<std::string>& argv,
     _stdLibDir = FileUtils::file_path(buffer, workingDir);
   } else if (cop.getOption("-G --globals-dir --mzn-globals-dir",
                            &_globalsDir)) {  // NOLINT: Allow repeated empty if
-    // Parsed by reference
+    // Resolve globals directory to be an absolute path
+    const std::string share = FileUtils::file_path(_stdLibDir + "/" + _globalsDir + "/");
+    const std::string rel = FileUtils::file_path(_globalsDir, workingDir);
+    if (FileUtils::is_absolute(_globalsDir)) {
+      // Do nothing (already have absolute path)
+    } else if ((_globalsDir.size() >= 2 && _globalsDir[0] == '.' && _globalsDir[1] == '/') ||
+               (_globalsDir.size() >= 2 && _globalsDir[0] == '.' && _globalsDir[1] == '\\') ||
+               (_globalsDir.size() >= 3 && _globalsDir[0] == '.' && _globalsDir[1] == '.' &&
+                _globalsDir[2] == '/') ||
+               (_globalsDir.size() >= 3 && _globalsDir[0] == '.' && _globalsDir[1] == '.' &&
+                _globalsDir[2] == '\\') ||
+               (FileUtils::directory_exists(rel) && !FileUtils::directory_exists(share))) {
+      _globalsDir = rel;
+    } else {
+      _globalsDir = share;
+    }
   } else if (cop.getOption("-D --cmdline-data", &buffer)) {
     _datafiles.push_back("cmd:/" + buffer);
   } else if (cop.getOption("--cmdline-json-data", &buffer)) {
@@ -390,6 +413,16 @@ bool Flattener::processOption(int& i, std::vector<std::string>& argv,
     }
     _log << "Error: model must have extension .mzn (or .fzn)" << std::endl;
     return false;
+  } else if (cop.getOption("--checker --solution-checker", &buffer)) {
+    if (buffer.length() <= 4) {
+      return false;
+    }
+    if (buffer.substr(buffer.length() - 4, string::npos) != ".mzc" &&
+        (buffer.length() <= 8 || buffer.substr(buffer.length() - 8, string::npos) != ".mzc.mzn")) {
+      _log << "Error: solution must have extension .mzc or .mzc.mzn" << std::endl;
+      return false;
+    }
+    _flagSolutionCheckModel = FileUtils::file_path(buffer, workingDir);
   } else {
     std::string input_file(argv[i]);
     if (input_file.length() <= 4) {
@@ -492,8 +525,7 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
   }
 
   if (!_globalsDir.empty()) {
-    _includePaths.insert(_includePaths.begin(),
-                         FileUtils::file_path(_stdLibDir + "/" + _globalsDir + "/"));
+    _includePaths.insert(_includePaths.begin(), _globalsDir);
   }
   _includePaths.push_back(FileUtils::file_path(_stdLibDir + "/std/"));
 
@@ -526,7 +558,14 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
 
   if (_fOutputByDefault) {
     if (_flagOutputFzn.empty()) {
-      _flagOutputFzn = _flagOutputBase + ".fzn";
+      switch (_flags.fznFormat) {
+        case FlattenerFlags::FF_FZN:
+          _flagOutputFzn = _flagOutputBase + ".fzn";
+          break;
+        case FlattenerFlags::FF_JSON:
+          _flagOutputFzn = _flagOutputBase + ".fzn.json";
+          break;
+      }
     }
     if (_flagOutputPaths.empty() && _fopts.collectMznPaths) {
       _flagOutputPaths = _flagOutputBase + ".paths";
@@ -551,8 +590,8 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
           _flagSolutionCheckModel.size() >= 4 &&
           _flagSolutionCheckModel.substr(_flagSolutionCheckModel.size() - 4) == ".mzc";
       std::vector<std::string> smm_model({_flagSolutionCheckModel});
-      Model* smm = parse(*env, smm_model, _datafiles, "", "", _includePaths, {}, _isFlatzinc, false,
-                         false, _flags.verbose, errstream);
+      Model* smm = parse(*env, smm_model, _datafiles, "", "", _includePaths, {}, _isFlatzinc,
+                         _flags.ignoreStdlib, false, _flags.verbose, errstream);
       if (_flags.verbose) {
         _log << " done parsing (" << _starttime.stoptime() << ")" << std::endl;
       }
@@ -561,7 +600,7 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
         errstream.str("");
         std::ostringstream smm_oss;
         std::ostringstream smm_stats_oss;
-        Printer p(smm_oss, 0, false);
+        Printer p(smm_oss, 0, false, &env->envi());
         p.print(smm);
         Env smm_env(smm);
         GCLock lock;
@@ -578,7 +617,8 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
             if (auto* vdi = i->dynamicCast<VarDeclI>()) {
               if (vdi->e()->type().isPar() && vdi->e()->e() == nullptr) {
                 env->envi().checkVars.emplace_back(vdi->e());
-              } else if (vdi->e()->ann().contains(Constants::constants().ann.rhs_from_assignment)) {
+              } else if (Expression::ann(vdi->e()).contains(
+                             Constants::constants().ann.rhs_from_assignment)) {
                 smm_stats_oss << *vdi;
               }
             }
@@ -587,10 +627,10 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
           std::string smm_compressed =
               FileUtils::encode_base64(FileUtils::deflate_string(smm_oss.str()));
           auto* ti = new TypeInst(Location().introduce(), Type::parstring(), nullptr);
+          auto* sl = new StringLit(Location().introduce(), smm_compressed);
           auto* checkString =
-              new VarDecl(Location().introduce(), ti, ASTString("_mzn_solution_checker"),
-                          new StringLit(Location().introduce(), smm_compressed));
-          auto* checkStringI = new VarDeclI(Location().introduce(), checkString);
+              new VarDecl(Location().introduce(), ti, ASTString("_mzn_solution_checker"), sl);
+          auto* checkStringI = VarDeclI::a(Location().introduce(), checkString);
           env->output()->addItem(checkStringI);
 
           for (FunctionIterator it = smm->functions().begin(); it != smm->functions().end(); ++it) {
@@ -606,10 +646,10 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
               std::string smm_stats_compressed =
                   FileUtils::encode_base64(FileUtils::deflate_string(smm_stats_oss.str()));
               auto* ti = new TypeInst(Location().introduce(), Type::parstring(), nullptr);
+              auto* sl = new StringLit(Location().introduce(), smm_stats_compressed);
               auto* checkStatsString =
-                  new VarDecl(Location().introduce(), ti, ASTString("_mzn_stats_checker"),
-                              new StringLit(Location().introduce(), smm_stats_compressed));
-              auto* checkStatsStringI = new VarDeclI(Location().introduce(), checkStatsString);
+                  new VarDecl(Location().introduce(), ti, ASTString("_mzn_stats_checker"), sl);
+              auto* checkStatsStringI = VarDeclI::a(Location().introduce(), checkStatsString);
               env->output()->addItem(checkStatsStringI);
             }
           }
@@ -667,8 +707,8 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
     errstream.str("");
     std::unordered_set<std::string> globalInc(global_includes(_stdLibDir));
     m = parse(*env, _filenames, _datafiles, modelText, modelName.empty() ? "stdin" : modelName,
-              _includePaths, std::move(globalInc), _isFlatzinc, false, false, _flags.verbose,
-              errstream);
+              _includePaths, std::move(globalInc), _isFlatzinc, _flags.ignoreStdlib, false,
+              _flags.verbose, errstream);
     if (!_globalsDir.empty()) {
       _includePaths.erase(_includePaths.begin());
     }
@@ -686,7 +726,7 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
           _flags.modelTypesOnly) {
         std::ostringstream compiledSolutionCheckModel;
         if (_flags.compileSolutionCheckModel) {
-          Printer p(compiledSolutionCheckModel, 0);
+          Printer p(compiledSolutionCheckModel, 0, true, &env->envi());
           type_demonomorphise_library(*env, m);
           p.print(m);
         }
@@ -732,6 +772,10 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
             throw MultipleErrors<TypeError>(typeErrors);
           }
           MiniZinc::register_builtins(*env);
+          env->model()->checkFnValid(env->envi(), typeErrors);
+          if (!typeErrors.empty()) {
+            throw MultipleErrors<TypeError>(typeErrors);
+          }
           env->swap();
           populate_output(*env, _flags.encapsulateJSON);
         } else {
@@ -746,6 +790,7 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
           _fopts.outputOutputItem = _flags.outputOutputItem;
           _fopts.hasChecker = !_flagSolutionCheckModel.empty();
           _fopts.encapsulateJSON = _flags.encapsulateJSON;
+          _fopts.ignoreStdlib = _flags.ignoreStdlib;
 #ifdef HAS_GECODE
           GecodeOptions gopts;
           gopts.onlyRangeDomains = _flags.onlyRangeDomains;
@@ -772,7 +817,7 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
 
           if (_flags.twoPass) {
             std::string library = _stdLibDir + (_flags.gecode ? "/gecode_presolver/" : "/std/");
-            bool differentLibrary = (library != _stdLibDir + "/" + _globalsDir + "/");
+            bool differentLibrary = (library != _globalsDir);
             managed_passes.emplace_back(new CompilePass(env, pass_opts, cfs, library, _includePaths,
                                                         true, differentLibrary));
 #ifdef HAS_GECODE
@@ -781,9 +826,8 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
             }
 #endif
           }
-          managed_passes.emplace_back(new CompilePass(env, _fopts, cfs,
-                                                      _stdLibDir + "/" + _globalsDir + "/",
-                                                      _includePaths, _flags.twoPass, false));
+          managed_passes.emplace_back(
+              new CompilePass(env, _fopts, cfs, _globalsDir, _includePaths, _flags.twoPass, false));
 
           Env* out_env = multiPassFlatten(managed_passes);
           if (out_env == nullptr) {
@@ -901,12 +945,12 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
             static void removePath(Annotation& a) {
               a.removeCall(Constants::constants().ann.mzn_path);
             }
-            static void vVarDeclI(VarDeclI* vdi) { removePath(vdi->e()->ann()); }
-            static void vConstraintI(ConstraintI* ci) { removePath(ci->e()->ann()); }
+            static void vVarDeclI(VarDeclI* vdi) { removePath(Expression::ann(vdi->e())); }
+            static void vConstraintI(ConstraintI* ci) { removePath(Expression::ann(ci->e())); }
             static void vSolveI(SolveI* si) {
               removePath(si->ann());
               if (Expression* e = si->e()) {
-                removePath(e->ann());
+                removePath(Expression::ann(e));
               }
             }
           } removePaths;
@@ -917,8 +961,16 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
           if (_flags.verbose) {
             _log << "Printing FlatZinc to stdout ..." << std::endl;
           }
-          Printer p(_os, 0);
-          p.print(env->flat());
+          switch (_flags.fznFormat) {
+            case FlattenerFlags::FF_FZN: {
+              Printer p(_os, 0, true, &env->envi());
+              p.print(env->flat());
+            } break;
+            case FlattenerFlags::FF_JSON: {
+              FznJSONPrinter p(_os, env->envi());
+              p.print(env->flat());
+            } break;
+          }
           if (_flags.verbose) {
             _log << " done (" << _starttime.stoptime() << ")" << std::endl;
           }
@@ -928,8 +980,16 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
           }
           std::ofstream ofs(FILE_PATH(_flagOutputFzn), ios::out);
           check_io_status(ofs.good(), " I/O error: cannot open fzn output file. ");
-          Printer p(ofs, 0);
-          p.print(env->flat());
+          switch (_flags.fznFormat) {
+            case FlattenerFlags::FF_FZN: {
+              Printer p(ofs, 0, true, &env->envi());
+              p.print(env->flat());
+            } break;
+            case FlattenerFlags::FF_JSON: {
+              FznJSONPrinter p(ofs, env->envi());
+              p.print(env->flat());
+            } break;
+          }
           check_io_status(ofs.good(), " I/O error: cannot write fzn output file. ");
           ofs.close();
           if (_flags.verbose) {
@@ -941,7 +1001,7 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
             if (_flags.verbose) {
               _log << "Printing .ozn to stdout ..." << std::endl;
             }
-            Printer p(_os, 0);
+            Printer p(_os, 0, true, &env->envi());
             Model* ozn;
             {
               GCLock lock;
@@ -958,7 +1018,7 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
             }
             std::ofstream ofs(FILE_PATH(_flagOutputOzn), std::ios::out);
             check_io_status(ofs.good(), " I/O error: cannot open ozn output file. ");
-            Printer p(ofs, 0);
+            Printer p(ofs, 0, true, &env->envi());
             Model* ozn;
             {
               GCLock lock;
@@ -975,7 +1035,7 @@ void Flattener::flatten(const std::string& modelString, const std::string& model
         }
       }
     } else {  // !flag_typecheck
-      Printer p(_os);
+      Printer p(_os, 80, true, &env->envi());
       p.print(m);
     }
   } catch (ResultUndefinedError& e) {
