@@ -74,7 +74,7 @@ protected:
   KeepAlive* _roots;
   WeakRef* _weakRefs;
   ASTNodeWeakMap* _nodeWeakMaps;
-  static const int _max_fl = 5;
+  static const int _max_fl = 9;
   FreeListNode* _fl[_max_fl + 1];
   static const size_t _fl_size[_max_fl + 1];
   static int freelistSlot(size_t _size) {
@@ -124,7 +124,7 @@ protected:
   }
 
   /// Default size of pages to allocate
-  static const size_t pageSize = 1 << 20;
+  static const size_t pageSize = 1 << 22;
 
   HeapPage* allocPage(size_t s, bool exact = false) {
     if (!exact) {
@@ -166,7 +166,7 @@ protected:
   }
 
   void* alloc(size_t size, bool exact = false) {
-    assert(size <= 80 || exact);
+    assert(size <= _fl_size[_max_fl] || exact);
     /// Align to word boundary
     size += ((8 - (size & 7)) & 7);
     HeapPage* p = _page;
@@ -260,6 +260,7 @@ protected:
         sizeof(AnonVar),        // E_ANON
         sizeof(ArrayLit),       // E_ARRAYLIT
         sizeof(ArrayAccess),    // E_ARRAYACCESS
+        sizeof(FieldAccess),    // E_ARRAYACCESS
         sizeof(Comprehension),  // E_COMP
         sizeof(ITE),            // E_ITE
         sizeof(BinOp),          // E_BINOP
@@ -270,7 +271,6 @@ protected:
         sizeof(TypeInst),       // E_TI
         sizeof(TIId),           // E_TIID
         sizeof(IncludeI),       // II_INC
-        sizeof(VarDeclI),       // II_VD
         sizeof(AssignI),        // II_ASN
         sizeof(ConstraintI),    // II_CON
         sizeof(SolveI),         // II_SOL
@@ -290,7 +290,26 @@ protected:
         ns = static_cast<ASTVec*>(n)->memsize();
         break;
       case Call::eid:
-        ns = n->_flag1 ? _nodesize[BinOp::eid] : _nodesize[Call::eid];
+        switch (static_cast<Call::CallKind>(n->_secondaryId)) {
+          case Call::CK_NULLARY:
+            ns = sizeof(Call0);
+            break;
+          case Call::CK_BINARY:
+          case Call::CK_NARY_2:
+            ns = sizeof(Call2);
+            break;
+          case Call::CK_TERNARY:
+          case Call::CK_NARY_3:
+            ns = sizeof(Call3);
+            break;
+          case Call::CK_QUATERNARY:
+          case Call::CK_NARY_4:
+            ns = sizeof(Call4);
+            break;
+          default:
+            ns = sizeof(CallNary);
+            break;
+        }
         break;
       default:
         assert(n->_id <= Item::II_END);
@@ -319,6 +338,7 @@ const char* GC::Heap::_nodeid[] = {
     "AnonVar       ",  // E_ANON
     "ArrayLit      ",  // E_ARRAYLIT
     "ArrayAccess   ",  // E_ARRAYACCESS
+    "FieldAccess   ",  // E_FIELDACCESS
     "Comprehension ",  // E_COMP
     "ITE           ",  // E_ITE
     "BinOp         ",  // E_BINOP
@@ -329,7 +349,6 @@ const char* GC::Heap::_nodeid[] = {
     "TypeInst      ",  // E_TI
     "TIId          ",  // E_TIID
     "IncludeI      ",  // II_INC
-    "VarDeclI      ",  // II_VD
     "AssignI       ",  // II_ASN
     "ConstraintI   ",  // II_CON
     "SolveI        ",  // II_SOL
@@ -364,6 +383,8 @@ const size_t GC::Heap::_fl_size[GC::Heap::_max_fl + 1] = {
     sizeof(Item) + 1 * sizeof(void*), sizeof(Item) + 2 * sizeof(void*),
     sizeof(Item) + 3 * sizeof(void*), sizeof(Item) + 4 * sizeof(void*),
     sizeof(Item) + 5 * sizeof(void*), sizeof(Item) + 6 * sizeof(void*),
+    sizeof(Item) + 7 * sizeof(void*), sizeof(Item) + 8 * sizeof(void*),
+    sizeof(Item) + 9 * sizeof(void*), sizeof(Item) + 10 * sizeof(void*),
 };
 
 GC::GC() : _heap(new Heap()), _lockCount(0) {}
@@ -398,7 +419,8 @@ void GC::remove(GCMarker* m) {
 void* GC::alloc(size_t size) {
   assert(locked());
   void* ret;
-  if (size < GC::Heap::_fl_size[0] || size > GC::Heap::_fl_size[GC::Heap::_max_fl]) {
+  size_t maxSize = GC::Heap::_fl_size[GC::Heap::_max_fl];
+  if (size < GC::Heap::_fl_size[0] || size > maxSize) {
     ret = _heap->alloc(size, true);
   } else {
     ret = _heap->fl(size);
@@ -438,23 +460,26 @@ void GC::Heap::mark() {
     Expression::mark(_trail[i].v);
   }
 
-  bool fixPrev = false;
+  m = _rootset;
+  do {
+    m->fixWeakRefs();
+    m = m->_rootsNext;
+  } while (m != _rootset);
+
   WeakRef* prevWr = nullptr;
   for (WeakRef* wr = _weakRefs; wr != nullptr; wr = wr->next()) {
-    if (fixPrev) {
-      fixPrev = false;
+    if (prevWr != nullptr) {
       removeWeakRef(prevWr);
       prevWr->_n = nullptr;
       prevWr->_p = nullptr;
+      prevWr = nullptr;
     }
     if (((*wr)() != nullptr) && (*wr)()->_gcMark == 0U) {
       wr->_e = nullptr;
-      wr->_valid = false;
-      fixPrev = true;
       prevWr = wr;
     }
   }
-  if (fixPrev) {
+  if (prevWr != nullptr) {
     removeWeakRef(prevWr);
     prevWr->_n = nullptr;
     prevWr->_p = nullptr;
@@ -484,6 +509,10 @@ void GC::Heap::sweep() {
 #endif
   HeapPage* p = _page;
   HeapPage* prev = nullptr;
+  std::vector<VarDecl*> fixVDGCMarks;
+  fixVDGCMarks.reserve(1000);
+  std::vector<HeapPage*> toFree;
+  toFree.reserve(1000);
   while (p != nullptr) {
     size_t off = 0;
     bool wholepage = true;
@@ -519,12 +548,13 @@ void GC::Heap::sweep() {
             break;
           case Expression::E_VARDECL:
             // Reset WeakRef inside VarDecl
+            n->_vdGcMark = 0U;
             static_cast<VarDecl*>(n)->flat(nullptr);
             // fall through
           default:
             if (n->_id >= static_cast<unsigned int>(ASTNode::NID_END) + 1 &&
                 n->_id <= static_cast<unsigned int>(Expression::EID_END)) {
-              static_cast<Expression*>(n)->ann().~Annotation();
+              Expression::ann(static_cast<Expression*>(n)).~Annotation();
             }
         }
         if (ns >= _fl_size[0] && ns <= _fl_size[_max_fl]) {
@@ -540,14 +570,19 @@ void GC::Heap::sweep() {
         wholepage = false;
         if (n->_id != static_cast<unsigned int>(ASTNode::NID_FL)) {
           n->_gcMark = 0;
+          if (n->_id == static_cast<unsigned int>(Expression::E_VARDECL)) {
+            auto* vd = static_cast<VarDecl*>(n);
+            if (vd->flat() != nullptr && vd->flat()->_vdGcMark == 0U) {
+              // the flat version has become garbage
+              vd->flat(nullptr);
+            }
+            fixVDGCMarks.push_back(vd);
+          }
         }
       }
       off += ns;
     }
     if (wholepage) {
-#ifndef NDEBUG
-      memset(p->data, 42, p->size);
-#endif
       if (prev != nullptr) {
         prev->next = p->next;
       } else {
@@ -561,7 +596,9 @@ void GC::Heap::sweep() {
         _freeMem -= (pf->size - pf->used);
       }
       assert(_allocedMem >= _freeMem);
-      ::free(pf);
+      // Can't call free() yet because we might free a VarDecl which is the flat version of one in
+      // another page
+      toFree.push_back(pf);
     } else {
       for (auto ni : freeNodes) {
         auto* fln = static_cast<FreeListNode*>(ni.n);
@@ -577,6 +614,15 @@ void GC::Heap::sweep() {
       p = p->next;
     }
   }
+  for (auto* vd : fixVDGCMarks) {
+    vd->_vdGcMark = 0U;
+  }
+  for (auto* pf : toFree) {
+#ifndef NDEBUG
+    memset(pf->data, 42, pf->size);
+#endif
+    ::free(pf);
+  }
 #if defined(MINIZINC_GC_STATS)
   for (auto stat : gc_stats) {
     std::cerr << _nodeid[stat.first] << ":\t" << stat.second.first << " / " << stat.second.second
@@ -664,29 +710,29 @@ void GC::removeKeepAlive(KeepAlive* e) {
 }
 
 KeepAlive::KeepAlive(Expression* e) : _e(e), _p(nullptr), _n(nullptr) {
-  if ((_e != nullptr) && !_e->isUnboxedVal()) {
+  if ((_e != nullptr) && !Expression::isUnboxedVal(_e)) {
     GC::gc()->addKeepAlive(this);
   }
 }
 KeepAlive::~KeepAlive() {
-  if ((_e != nullptr) && !_e->isUnboxedVal()) {
+  if ((_e != nullptr) && !Expression::isUnboxedVal(_e)) {
     GC::gc()->removeKeepAlive(this);
   }
 }
 KeepAlive::KeepAlive(const KeepAlive& e) : _e(e._e), _p(nullptr), _n(nullptr) {
-  if ((_e != nullptr) && !_e->isUnboxedVal()) {
+  if ((_e != nullptr) && !Expression::isUnboxedVal(_e)) {
     GC::gc()->addKeepAlive(this);
   }
 }
 KeepAlive& KeepAlive::operator=(const KeepAlive& e) {
   if (this != &e) {
-    if ((_e != nullptr) && !_e->isUnboxedVal()) {
-      if (e._e == nullptr || e._e->isUnboxedVal()) {
+    if ((_e != nullptr) && !Expression::isUnboxedVal(_e)) {
+      if (e._e == nullptr || Expression::isUnboxedVal(e._e)) {
         GC::gc()->removeKeepAlive(this);
         _p = _n = nullptr;
       }
     } else {
-      if (e._e != nullptr && !e._e->isUnboxedVal()) {
+      if (e._e != nullptr && !Expression::isUnboxedVal(e._e)) {
         GC::gc()->addKeepAlive(this);
       }
     }
@@ -736,44 +782,50 @@ void GC::removeNodeWeakMap(ASTNodeWeakMap* m) {
   }
 }
 
-WeakRef::WeakRef(Expression* e) : _e(e), _p(nullptr), _n(nullptr), _valid(true) {
-  if ((_e != nullptr) && !_e->isUnboxedVal()) {
+WeakRef::WeakRef(Expression* e) : _e(e), _p(nullptr), _n(nullptr) {
+  if (_e != nullptr && !Expression::isUnboxedVal(_e)) {
     GC::gc()->addWeakRef(this);
   }
 }
 WeakRef::~WeakRef() {
-  if ((_e != nullptr) && !_e->isUnboxedVal()) {
+  if ((_e != nullptr) && !Expression::isUnboxedVal(_e)) {
     GC::gc()->removeWeakRef(this);
   }
 }
-WeakRef::WeakRef(const WeakRef& e) : _e(e()), _p(nullptr), _n(nullptr), _valid(true) {
-  if ((_e != nullptr) && !_e->isUnboxedVal()) {
+WeakRef::WeakRef(const WeakRef& e) : _e(e()), _p(nullptr), _n(nullptr) {
+  if (_e != nullptr && !Expression::isUnboxedVal(_e)) {
     GC::gc()->addWeakRef(this);
   }
 }
 WeakRef& WeakRef::operator=(const WeakRef& e) {
   if (this != &e) {
     // Test if this WeakRef is currently active in the GC
-    bool isActive = ((_e != nullptr) && !_e->isUnboxedVal());
+    bool isActive = ((_e != nullptr) && !Expression::isUnboxedVal(_e));
     if (isActive) {
       // Yes, active WeakRef.
       // If after assigning WeakRef should be inactive, remove it.
-      if (e() == nullptr || e()->isUnboxedVal()) {
+      if (e() == nullptr || Expression::isUnboxedVal(e())) {
         GC::gc()->removeWeakRef(this);
         _n = _p = nullptr;
       }
     }
     _e = e();
-    _valid = true;
 
     // If this WeakRef was not active but now should be, add it
-    if (!isActive && _e != nullptr && !_e->isUnboxedVal()) {
+    if (!isActive && _e != nullptr && !Expression::isUnboxedVal(_e)) {
       GC::gc()->addWeakRef(this);
     }
   }
   return *this;
 }
 
+Expression* WeakRef::operator()() {
+  return Expression::isUnboxedVal(_e) ? _e : _p != nullptr ? _e : nullptr;
+}
+Expression* WeakRef::operator()() const {
+  return Expression::isUnboxedVal(_e) ? _e : _p != nullptr ? _e : nullptr;
+}
+
 ASTNodeWeakMap::ASTNodeWeakMap() : _p(nullptr), _n(nullptr) { GC::gc()->addNodeWeakMap(this); }
 
 ASTNodeWeakMap::~ASTNodeWeakMap() { GC::gc()->removeNodeWeakMap(this); }
