@@ -33,6 +33,20 @@ namespace MiniZinc {
 using std::string;
 using std::vector;
 
+ASTString strip_stdlib_path(const vector<string>& includePaths, const ASTString& fs) {
+  std::string f(fs.c_str());
+  for (const auto& p : includePaths) {
+    if (f.size() > p.size() && f.substr(0, p.size()) == p) {
+      f = f.substr(p.size());
+      while (!f.empty() && f[0] == '/') {
+        f = f.substr(1);
+      }
+      return ASTString(f);
+    }
+  }
+  return fs;
+}
+
 Env* change_library(Env& e, vector<string>& includePaths, const string& globals_dir,
                     CompilePassFlags& compflags, bool verbose = false) {
   GCLock lock;
@@ -53,7 +67,11 @@ Env* change_library(Env& e, vector<string>& includePaths, const string& globals_
   vector<ASTString> include_names;
   for (Item* item : *m) {
     if (auto* inc = item->dynamicCast<IncludeI>()) {
-      include_names.push_back(inc->m()->filepath());
+      if (FileUtils::is_absolute(inc->f().c_str())) {
+        include_names.push_back(inc->f());
+      } else {
+        include_names.push_back(strip_stdlib_path(new_includePaths, inc->m()->filepath()));
+      }
     } else {
       new_mod->addItem(copy(e.envi(), cm, item));
     }
@@ -64,23 +82,15 @@ Env* change_library(Env& e, vector<string>& includePaths, const string& globals_
     ss << "include \"" << Printer::escapeStringLit(name) << "\";";
   }
 
-  vector<SyntaxError> syntax_errors;
   Env* fenv = new Env(new_mod);
   // Model* inc_mod = parse(*fenv, include_names, {}, new_includePaths, true, true, verbose,
   // std::cerr);
   std::ostringstream dummy_file;
   dummy_file << m->filepath() << "_Dummy.mzn";
   Model* inc_mod = parse_from_string(*fenv, ss.str(), dummy_file.str(), new_includePaths, false,
-                                     false, true, verbose, std::cerr, syntax_errors);
-  if (inc_mod == nullptr) {
-    for (const SyntaxError& se : syntax_errors) {
-      std::cerr << std::endl;
-      std::cerr << se.what() << ": " << se.msg() << std::endl;
-      std::cerr << se.loc() << std::endl;
-    }
-    return nullptr;
-  }
-  auto* new_inc = new IncludeI(Location().introduce(), string("MultiPassDummy.mzn"));
+                                     false, true, verbose, std::cerr);
+
+  auto* new_inc = new IncludeI(Location().introduce(), ASTString("MultiPassDummy.mzn"));
   new_inc->m(inc_mod);
   inc_mod->setParent(new_mod);
   new_mod->addItem(new_inc);
@@ -122,15 +132,14 @@ Env* CompilePass::run(Env* store, std::ostream& log) {
                       _compflags.modelCheckOnly || _compflags.modelInterfaceOnly,
                       _compflags.allowMultiAssign);
   if (!typeErrors.empty()) {
-    std::ostringstream errstream;
-    for (auto& typeError : typeErrors) {
-      errstream << typeError.what() << ": " << typeError.msg() << std::endl;
-      errstream << typeError.loc() << std::endl;
-    }
-    throw Error(errstream.str());
+    throw MultipleErrors<TypeError>(typeErrors);
   }
 
   register_builtins(*new_env);
+  new_env->model()->checkFnValid(_env->envi(), typeErrors);
+  if (!typeErrors.empty()) {
+    throw MultipleErrors<TypeError>(typeErrors);
+  }
 
   try {
     flatten(*new_env, _fopts);
@@ -138,18 +147,15 @@ Env* CompilePass::run(Env* store, std::ostream& log) {
     if (_compflags.verbose) {
       log << std::endl;
     }
-    std::ostringstream errstream;
-    errstream << e.what() << ": " << std::endl;
-    new_env->dumpErrorStack(errstream);
-    errstream << "  " << e.msg() << std::endl;
-    throw Error(errstream.str());
+    e.dumpStack(true);
+    throw;
   }
 
   if (!_compflags.noMIPdomains) {
     if (_compflags.verbose) {
-      log << "MIP domains ...";
+      log << "MIP domains ..." << std::endl;
     }
-    mip_domains(*new_env, _compflags.statistics);
+    mip_domains(*new_env, _compflags.verbose);
     if (_compflags.verbose) {
       log << " done (" << lasttime.stoptime() << ")" << std::endl;
     }
@@ -164,19 +170,20 @@ Env* CompilePass::run(Env* store, std::ostream& log) {
       log << " done (" << lasttime.stoptime() << ")" << std::endl;
     }
   }
+  if (!new_env->warnings().empty() && _compflags.werror) {
+    // Flattener will print warnings
+    throw Error("warnings treated as errors.");
+  }
 
-  for (const auto& i : new_env->warnings()) {
-    log << (_compflags.werror ? "\n  ERROR: " : "\n  WARNING: ") << i;
-  }
-  if (_compflags.werror && !new_env->warnings().empty()) {
-    throw Error("errors encountered");
-  }
+  new_env->dumpWarnings(_fopts.encapsulateJSON ? std::cout : log, _compflags.werror,
+                        _fopts.encapsulateJSON);
   new_env->clearWarnings();
 
   if (!_compflags.newfzn) {
     if (_compflags.verbose) {
       log << "Converting to old FlatZinc ...";
     }
+    substitute_fixed_vars(*new_env);
     oldflatzinc(*new_env);
     if (_compflags.verbose) {
       log << " done (" << lasttime.stoptime() << ")" << std::endl;
