@@ -88,25 +88,25 @@ string path2name(const string& path) {
   return name.str();
 }
 
-void PathFilePrinter::print(Model* m) {
+void PathFilePrinter::buildMap(Model* m) {
   // Build map
   for (VarDeclIterator vdit = m->vardecls().begin(); vdit != m->vardecls().end(); ++vdit) {
     VarDecl* e = vdit->e();
-    for (ExpressionSetIter it = e->ann().begin(); it != e->ann().end(); ++it) {
-      if (Call* ca = (*it)->dynamicCast<Call>()) {
+    for (ExpressionSetIter it = Expression::ann(e).begin(); it != Expression::ann(e).end(); ++it) {
+      if (Call* ca = Expression::dynamicCast<Call>(*it)) {
         ASTString cid = ca->id();
-        if (cid == constants().ann.output_array) {
-          if (auto* rhs = e->e()->dynamicCast<ArrayLit>()) {
+        if (cid == Constants::constants().ann.output_array) {
+          if (auto* rhs = Expression::dynamicCast<ArrayLit>(e->e())) {
             for (unsigned int ind = 0; ind < rhs->size(); ind++) {
-              if (Id* id = (*rhs)[ind]->dynamicCast<Id>()) {
+              if (Id* id = Expression::dynamicCast<Id>((*rhs)[ind])) {
                 std::stringstream bettername;
                 bettername << *e->id() << "[";
 
                 // Array of sets
-                ArrayLit& dimsets = *ca->arg(0)->cast<ArrayLit>();
+                ArrayLit& dimsets = *Expression::cast<ArrayLit>(ca->arg(0));
                 vector<IntVal> dims(dimsets.size(), 1);
                 for (unsigned int i = 0; i < dimsets.size(); i++) {
-                  auto* sl = dimsets[i]->cast<SetLit>();
+                  auto* sl = Expression::cast<SetLit>(dimsets[i]);
                   dims[i] = sl->isv()->card();
                 }
                 vector<IntVal> dimspan(dims.size(), 1);
@@ -120,24 +120,28 @@ void PathFilePrinter::print(Model* m) {
                 for (unsigned int i = 0; i < dims.size() - 1; i++) {
                   IntVal thisind = curind / dimspan[i];
                   curind -= thisind * dimspan[i];
-                  bettername << dimsets[i]->cast<SetLit>()->isv()->min() + thisind << ",";
+                  bettername << Expression::cast<SetLit>(dimsets[i])->isv()->min() + thisind << ",";
                 }
-                bettername << dimsets[dimsets.size() - 1]->cast<SetLit>()->isv()->min() + curind
+                bettername << Expression::cast<SetLit>(dimsets[dimsets.size() - 1])->isv()->min() +
+                                  curind
                            << "]";
 
                 addBetterName(id, bettername.str(), "", true);
               }
             }
           }
-        } else if (ca->id() == constants().ann.mzn_path) {
-          auto* sl = ca->arg(0)->cast<StringLit>();
+        } else if (ca->id() == Constants::constants().ann.mzn_path) {
+          auto* sl = Expression::cast<StringLit>(ca->arg(0));
           addBetterName(e->id(), path2name(string(sl->v().c_str(), sl->v().size())),
                         string(sl->v().c_str()));
         }
       }
     }
   }
+}
 
+void PathFilePrinter::print(Model* m) {
+  buildMap(m);
   // Print values
   for (Item* item : *m) {
     print(item);
@@ -170,11 +174,11 @@ void PathFilePrinter::print(Item* item) {
   } else if (auto* ci = item->dynamicCast<ConstraintI>()) {
     StringLit* sl = nullptr;
     Expression* e = ci->e();
-    for (ExpressionSetIter it = e->ann().begin(); it != e->ann().end(); ++it) {
-      if (Call* ca = (*it)->dynamicCast<Call>()) {
+    for (ExpressionSetIter it = Expression::ann(e).begin(); it != Expression::ann(e).end(); ++it) {
+      if (Call* ca = Expression::dynamicCast<Call>(*it)) {
         ASTString cid = ca->id();
-        if (cid == constants().ann.mzn_path) {
-          sl = ca->arg(0)->cast<StringLit>();
+        if (cid == Constants::constants().ann.mzn_path) {
+          sl = Expression::cast<StringLit>(ca->arg(0));
         }
       }
     }
@@ -190,4 +194,77 @@ void PathFilePrinter::print(Item* item) {
     _constraintIndex++;
   }
 }
+
+void PathFilePrinter::json(Model* m) {
+  buildMap(m);
+  // Print values
+  _os << "[";
+  bool first = true;
+  for (Item* item : *m) {
+    GCLock lock;
+    if (auto* vdi = item->dynamicCast<VarDeclI>()) {
+      Id* id = vdi->e()->id();
+      NamePair np = _betternames[id];
+      if (!np.first.empty() || !np.second.empty()) {
+        if (first) {
+          first = false;
+        } else {
+          _os << ", ";
+        }
+        _os << "{";
+        // FlatZinc name
+        _os << "\"flatZincName\": \"" << Printer::escapeStringLit(id->str()) << "\", ";
+
+        // Nice name
+        _os << "\"niceName\": \"";
+        if (np.first.empty()) {
+          _os << Printer::escapeStringLit(id->str());
+        } else {
+          string name = np.first;
+          _os << Printer::escapeStringLit(name);
+          if (name.find('?') != string::npos) {
+            _os << "(" << Printer::escapeStringLit(id->str()) << ")";
+          }
+        }
+        _os << "\", ";
+
+        // Path
+        _os << "\"path\": \"" << Printer::escapeStringLit(np.second) << "\"";
+        _os << "}";
+      }
+    } else if (auto* ci = item->dynamicCast<ConstraintI>()) {
+      StringLit* sl = nullptr;
+      Expression* e = ci->e();
+      for (ExpressionSetIter it = Expression::ann(e).begin(); it != Expression::ann(e).end();
+           ++it) {
+        if (Call* ca = Expression::dynamicCast<Call>(*it)) {
+          ASTString cid = ca->id();
+          if (cid == Constants::constants().ann.mzn_path) {
+            sl = Expression::cast<StringLit>(ca->arg(0));
+          }
+        }
+      }
+      if (first) {
+        first = false;
+      } else {
+        _os << ", ";
+      }
+      _os << "{";
+      _os << "\"constraintIndex\": " << _constraintIndex << ", ";
+      _os << "\"path\": ";
+      if (sl != nullptr) {
+        _os << "\"" << Printer::escapeStringLit(sl->v()) << "\"";
+      } else {
+        _os << "null";
+      }
+      _constraintIndex++;
+      _os << "}";
+    }
+    _os.flush();
+    assert(_os.good());
+  }
+  _os << "]";
+  assert(_os.good());
+}
+
 }  // namespace MiniZinc
