@@ -9,6 +9,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include <minizinc/astexception.hh>
 #include <minizinc/file_utils.hh>
 #include <minizinc/json_parser.hh>
 #include <minizinc/parser.hh>
@@ -26,6 +27,21 @@ using namespace std;
 
 namespace MiniZinc {
 
+void json_coerce_assignments_2d(JSONParser& jp, Model* m, const std::vector<std::string>& names) {
+  for (auto& i : *m) {
+    if (auto* ai = i->dynamicCast<AssignI>()) {
+      std::string ident(ai->id().c_str(), ai->id().size());
+      if (std::find(names.begin(), names.end(), ident) != names.end()) {
+        if (auto* al = Expression::dynamicCast<ArrayLit>(ai->e())) {
+          GCLock lock;
+          auto* ti = new TypeInst(Location().introduce(), Type::mkAny(2));
+          ai->e(jp.coerceArray(ti, al));
+        }
+      }
+    }
+  }
+}
+
 bool SolverConfig::ExtraFlag::validate(const std::string& v) const {
   try {
     switch (flagType) {
@@ -51,28 +67,28 @@ bool SolverConfig::ExtraFlag::validate(const std::string& v) const {
 
 namespace {
 std::string get_string(AssignI* ai) {
-  if (auto* sl = ai->e()->dynamicCast<StringLit>()) {
+  if (auto* sl = Expression::dynamicCast<StringLit>(ai->e())) {
     return std::string(sl->v().c_str(), sl->v().size());
   }
   throw ConfigException("invalid configuration item (right hand side must be string)");
 }
 bool get_bool(AssignI* ai) {
-  if (auto* bl = ai->e()->dynamicCast<BoolLit>()) {
+  if (auto* bl = Expression::dynamicCast<BoolLit>(ai->e())) {
     return bl->v();
   }
   throw ConfigException("invalid configuration item (right hand side must be bool)");
 }
 int get_int(AssignI* ai) {
-  if (auto* il = ai->e()->dynamicCast<IntLit>()) {
-    return static_cast<int>(il->v().toInt());
+  if (auto* il = Expression::dynamicCast<IntLit>(ai->e())) {
+    return static_cast<int>(IntLit::v(il).toInt());
   }
   throw ConfigException("invalid configuration item (right hand side must be int)");
 }
 std::vector<std::string> get_string_list(AssignI* ai) {
-  if (auto* al = ai->e()->dynamicCast<ArrayLit>()) {
+  if (auto* al = Expression::dynamicCast<ArrayLit>(ai->e())) {
     std::vector<std::string> ret;
     for (unsigned int i = 0; i < al->size(); i++) {
-      if (auto* sl = (*al)[i]->dynamicCast<StringLit>()) {
+      if (auto* sl = Expression::dynamicCast<StringLit>((*al)[i])) {
         ret.emplace_back(sl->v().c_str(), sl->v().size());
       } else {
         throw ConfigException(
@@ -84,15 +100,15 @@ std::vector<std::string> get_string_list(AssignI* ai) {
   throw ConfigException("invalid configuration item (right hand side must be a list of strings)");
 }
 std::vector<std::pair<std::string, std::string> > get_string_pair_list(AssignI* ai) {
-  if (auto* al = ai->e()->dynamicCast<ArrayLit>()) {
+  if (auto* al = Expression::dynamicCast<ArrayLit>(ai->e())) {
     std::vector<std::pair<std::string, std::string> > ret;
     if (al->dims() != 2 || al->min(1) != 1 || al->max(1) != 2) {
       throw ConfigException(
           "invalid configuration item (right hand side must be a 2d array of strings)");
     }
     for (unsigned int i = 0; i < al->size(); i += 2) {
-      auto* sl1 = (*al)[i]->dynamicCast<StringLit>();
-      auto* sl2 = (*al)[i + 1]->dynamicCast<StringLit>();
+      auto* sl1 = Expression::dynamicCast<StringLit>((*al)[i]);
+      auto* sl2 = Expression::dynamicCast<StringLit>((*al)[i + 1]);
       if ((sl1 != nullptr) && (sl2 != nullptr)) {
         ret.emplace_back(std::string(sl1->v().c_str(), sl1->v().size()),
                          std::string(sl2->v().c_str(), sl2->v().size()));
@@ -107,9 +123,9 @@ std::vector<std::pair<std::string, std::string> > get_string_pair_list(AssignI*
       "invalid configuration item (right hand side must be a 2d array of strings)");
 }
 std::vector<std::vector<std::string> > get_default_option_list(AssignI* ai) {
-  if (auto* al = ai->e()->dynamicCast<ArrayLit>()) {
+  if (auto* al = Expression::dynamicCast<ArrayLit>(ai->e())) {
     std::vector<std::vector<std::string> > ret;
-    if (al->size() == 0) {
+    if (al->empty()) {
       return ret;
     }
     if (al->dims() != 2) {
@@ -123,9 +139,9 @@ std::vector<std::vector<std::string> > get_default_option_list(AssignI* ai) {
           "columns)");
     }
     for (unsigned int i = 0; i < al->size(); i += nCols) {
-      auto* sl0 = (*al)[i]->dynamicCast<StringLit>();
-      auto* sl1 = (*al)[i + 1]->dynamicCast<StringLit>();
-      auto* sl2 = (*al)[i + 2]->dynamicCast<StringLit>();
+      auto* sl0 = Expression::dynamicCast<StringLit>((*al)[i]);
+      auto* sl1 = Expression::dynamicCast<StringLit>((*al)[i + 1]);
+      auto* sl2 = Expression::dynamicCast<StringLit>((*al)[i + 2]);
       if ((sl0 != nullptr) && (sl1 != nullptr) && (sl2 != nullptr)) {
         ret.push_back(std::vector<std::string>({std::string(sl0->v().c_str(), sl0->v().size()),
                                                 std::string(sl1->v().c_str(), sl1->v().size()),
@@ -141,9 +157,9 @@ std::vector<std::vector<std::string> > get_default_option_list(AssignI* ai) {
       "invalid configuration item (right hand side must be a 2d array of strings)");
 }
 std::vector<SolverConfig::ExtraFlag> get_extra_flag_list(AssignI* ai) {
-  if (auto* al = ai->e()->dynamicCast<ArrayLit>()) {
+  if (auto* al = Expression::dynamicCast<ArrayLit>(ai->e())) {
     std::vector<SolverConfig::ExtraFlag> ret;
-    if (al->size() == 0) {
+    if (al->empty()) {
       return ret;
     }
     if (al->dims() != 2) {
@@ -158,10 +174,10 @@ std::vector<SolverConfig::ExtraFlag> get_extra_flag_list(AssignI* ai) {
     bool haveType = (nCols >= 3);
     bool haveDefault = (nCols >= 4);
     for (unsigned int i = 0; i < al->size(); i += nCols) {
-      auto* sl1 = (*al)[i]->dynamicCast<StringLit>();
-      auto* sl2 = (*al)[i + 1]->dynamicCast<StringLit>();
-      StringLit* sl3 = haveType ? (*al)[i + 2]->dynamicCast<StringLit>() : nullptr;
-      StringLit* sl4 = haveDefault ? (*al)[i + 3]->dynamicCast<StringLit>() : nullptr;
+      auto* sl1 = Expression::dynamicCast<StringLit>((*al)[i]);
+      auto* sl2 = Expression::dynamicCast<StringLit>((*al)[i + 1]);
+      StringLit* sl3 = haveType ? Expression::dynamicCast<StringLit>((*al)[i + 2]) : nullptr;
+      StringLit* sl4 = haveDefault ? Expression::dynamicCast<StringLit>((*al)[i + 3]) : nullptr;
       std::string opt_type_full =
           sl3 != nullptr ? std::string(sl3->v().c_str(), sl3->v().size()) : "bool";
       std::vector<std::string> opt_range;
@@ -211,28 +227,6 @@ std::vector<SolverConfig::ExtraFlag> get_extra_flag_list(AssignI* ai) {
       "invalid configuration item (right hand side must be a 2d array of strings)");
 }
 
-std::string get_env(const char* v) {
-  std::string ret;
-#ifdef _MSC_VER
-  size_t len;
-  getenv_s(&len, nullptr, 0, v);
-  if (len > 0) {
-    char* p = static_cast<char*>(malloc(len * sizeof(char)));
-    getenv_s(&len, p, len, v);
-    if (len > 0) {
-      ret = p;
-    }
-    free(p);
-  }
-#else
-  char* p = getenv(v);
-  if (p != nullptr) {
-    ret = p;
-  }
-#endif
-  return ret;
-}
-
 char char_to_lower(char c) {
   return static_cast<char>(std::tolower(static_cast<unsigned char>(c)));
 }
@@ -274,6 +268,7 @@ SolverConfig SolverConfig::load(const string& filename) {
         m = new Model;
         GCLock lock;
         jp.parse(m, filename, false);
+        json_coerce_assignments_2d(jp, m, {"extraFlags"});
       } catch (JSONError& e) {
         delete m;
         m = nullptr;
@@ -282,8 +277,7 @@ SolverConfig SolverConfig::load(const string& filename) {
     } else {
       vector<string> filenames;
       filenames.push_back(filename);
-      m = parse(confenv, filenames, vector<string>(), "", "", vector<string>(), false, true, false,
-                false, errstream);
+      m = parse(confenv, filenames, {}, "", "", {}, {}, false, true, false, false, errstream);
     }
     if (m != nullptr) {
       bool hadId = false;
@@ -299,11 +293,23 @@ SolverConfig SolverConfig::load(const string& filename) {
             sc._name = get_string(ai);
             hadName = true;
           } else if (ai->id() == "executable") {
-            std::string exePath = get_string(ai);
+            std::string exePath;
+            if (Expression::isa<ArrayLit>(ai->e())) {
+              std::vector<std::string> exeArgs(get_string_list(ai));
+              if (!exeArgs.empty()) {
+                exePath = exeArgs[0];
+                sc._passFlags.reserve(exeArgs.size() - 1);
+                for (size_t i = 1; i < exeArgs.size(); ++i) {
+                  sc._passFlags.push_back(exeArgs[i]);
+                }
+              }
+            } else {
+              exePath = get_string(ai);
+            }
             sc._executable = exePath;
-            std::string exe = FileUtils::find_executable(FileUtils::file_path(exePath, basePath));
+            std::string exe = FileUtils::find_executable(exePath, basePath);
             int nr_found = (int)(!exe.empty());
-            std::string tmp = FileUtils::file_path(FileUtils::find_executable(exePath));
+            std::string tmp = FileUtils::find_executable(exePath);
             nr_found += (int)((!tmp.empty()) && tmp != exe);
             exe = exe.empty() ? tmp : exe;
             if (nr_found > 0) {
@@ -335,11 +341,33 @@ SolverConfig SolverConfig::load(const string& filename) {
           } else if (ai->id() == "website") {
             sc._website = get_string(ai);
           } else if (ai->id() == "supportsMzn") {
-            sc._supportsMzn = get_bool(ai);
+            if (get_bool(ai)) {
+              sc._inputType = O_MZN;
+            }
           } else if (ai->id() == "supportsFzn") {
-            sc._supportsFzn = get_bool(ai);
+            if (get_bool(ai)) {
+              sc._inputType = O_FZN;
+            }
           } else if (ai->id() == "supportsNL") {
-            sc._supportsNL = get_bool(ai);
+            if (get_bool(ai)) {
+              sc._inputType = O_NL;
+            }
+          } else if (ai->id() == "inputType") {
+            std::string str = get_string(ai);
+            std::transform(str.begin(), str.end(), str.begin(), ::toupper);
+            if (str == "FZN") {
+              sc._inputType = O_FZN;
+            } else if (str == "MZN") {
+              sc._inputType = O_MZN;
+            } else if (str == "NL") {
+              sc._inputType = O_NL;
+            } else if (str == "JSON") {
+              sc._inputType = O_JSON;
+            } else {
+              std::ostringstream ss;
+              ss << "unknown input type (" << str << ")";
+              throw ConfigException(ss.str());
+            }
           } else if (ai->id() == "needsSolns2Out") {
             sc._needsSolns2Out = get_bool(ai);
           } else if (ai->id() == "isGUIApplication") {
@@ -496,9 +524,22 @@ std::string SolverConfig::toJSON(const SolverConfigs& configs) const {
     }
     oss << "],\n";
   }
-  oss << "  \"supportsMzn\": " << (supportsMzn() ? "true" : "false") << ",\n";
-  oss << "  \"supportsFzn\": " << (supportsFzn() ? "true" : "false") << ",\n";
-  oss << "  \"supportsNL\": " << (supportsNL() ? "true" : "false") << ",\n";
+  oss << "  \"inputType\": \"";
+  switch (inputType()) {
+    case O_FZN:
+      oss << "FZN";
+      break;
+    case O_MZN:
+      oss << "MZN";
+      break;
+    case O_NL:
+      oss << "NL";
+      break;
+    case O_JSON:
+      oss << "JSON";
+      break;
+  }
+  oss << "\",\n";
   oss << "  \"needsSolns2Out\": " << (needsSolns2Out() ? "true" : "false") << ",\n";
   oss << "  \"needsMznExecutable\": " << (needsMznExecutable() ? "true" : "false") << ",\n";
   oss << "  \"needsStdlibDir\": " << (needsStdlibDir() ? "true" : "false") << ",\n";
@@ -547,22 +588,8 @@ void SolverConfigs::addConfig(const MiniZinc::SolverConfig& sc) {
 std::vector<std::string> SolverConfigs::solverConfigsPath() const { return _solverPath; }
 
 SolverConfigs::SolverConfigs(std::ostream& log) {
-#ifdef _MSC_VER
-  const char* PATHSEP = ";";
-#else
-  const char* PATHSEP = ":";
-#endif
-  std::string mzn_solver_path = get_env("MZN_SOLVER_PATH");
-  while (!mzn_solver_path.empty()) {
-    size_t next_sep = mzn_solver_path.find(PATHSEP);
-    string cur_path = mzn_solver_path.substr(0, next_sep);
-    _solverPath.push_back(cur_path);
-    if (next_sep != string::npos) {
-      mzn_solver_path = mzn_solver_path.substr(next_sep + 1, string::npos);
-    } else {
-      mzn_solver_path = "";
-    }
-  }
+  auto paths = FileUtils::get_env_list("MZN_SOLVER_PATH");
+  _solverPath.insert(_solverPath.end(), paths.begin(), paths.end());
   std::string userConfigDir = FileUtils::user_config_dir();
   if (FileUtils::directory_exists(userConfigDir + "/solvers")) {
     _solverPath.push_back(userConfigDir + "/solvers");
@@ -575,78 +602,65 @@ SolverConfigs::SolverConfigs(std::ostream& log) {
       ostringstream errstream;
       try {
         Env userconfenv;
-        Model* m = nullptr;
-        if (JSONParser::fileIsJSON(cf)) {
-          JSONParser jp(userconfenv.envi());
-          try {
-            m = new Model;
-            GCLock lock;
-            jp.parse(m, cf, false);
-          } catch (JSONError&) {
-            delete m;
-            m = nullptr;
-          }
+        if (!JSONParser::fileIsJSON(cf)) {
+          throw ConfigException(cf + "is not a JSON file.");
         }
-        if (m != nullptr) {
-          for (auto& i : *m) {
-            if (auto* ai = i->dynamicCast<AssignI>()) {
-              if (ai->id() == "mzn_solver_path") {
-                std::vector<std::string> sp = get_string_list(ai);
-                for (const auto& s : sp) {
-                  _solverPath.push_back(s);
-                }
-              } else if (ai->id() == "mzn_lib_dir") {
-                _mznlibDir = get_string(ai);
-              } else if (ai->id() == "tagDefaults") {
-                std::vector<std::pair<std::string, std::string> > tagDefs =
-                    get_string_pair_list(ai);
-                for (auto& td : tagDefs) {
-                  std::string tag = td.first;
-                  std::string solver_id = td.second;
-                  _tagDefault[tag] = solver_id;
-                }
-              } else if (ai->id() == "solverDefaults") {
-                std::vector<std::vector<std::string> > solverDefs = get_default_option_list(ai);
-                for (auto& sd : solverDefs) {
-                  assert(sd.size() == 3);
-                  std::string solver = sd[0];
-                  auto it = _solverDefaultOptions.find(solver);
-                  if (it == _solverDefaultOptions.end()) {
-                    std::vector<std::string> solverOptions({sd[1], sd[2]});
-                    _solverDefaultOptions.insert(std::make_pair(solver, solverOptions));
-                  } else {
-                    std::vector<std::string>& opts = it->second;
-                    bool found = false;
-                    for (unsigned int i = 0; i < opts.size(); i += 2) {
-                      if (opts[i] == sd[1]) {
-                        // Override existing option value
-                        opts[i + 1] = sd[2];
-                        found = true;
-                        break;
-                      }
-                    }
-                    if (!found) {
-                      // Option didn't exist, add to end of list
-                      opts.push_back(sd[1]);
-                      opts.push_back(sd[2]);
+        GCLock lock;
+        JSONParser jp(userconfenv.envi());
+        Model m;
+        jp.parse(&m, cf, false);
+        json_coerce_assignments_2d(jp, &m, {"tagDefaults", "solverDefaults"});
+        for (auto& i : m) {
+          if (auto* ai = i->dynamicCast<AssignI>()) {
+            if (ai->id() == "mzn_solver_path") {
+              std::vector<std::string> sp = get_string_list(ai);
+              for (const auto& s : sp) {
+                _solverPath.push_back(s);
+              }
+            } else if (ai->id() == "mzn_lib_dir") {
+              _mznlibDir = get_string(ai);
+            } else if (ai->id() == "tagDefaults") {
+              std::vector<std::pair<std::string, std::string> > tagDefs = get_string_pair_list(ai);
+              for (auto& td : tagDefs) {
+                std::string tag = td.first;
+                std::string solver_id = td.second;
+                _tagDefault[tag] = solver_id;
+              }
+            } else if (ai->id() == "solverDefaults") {
+              std::vector<std::vector<std::string> > solverDefs = get_default_option_list(ai);
+              for (auto& sd : solverDefs) {
+                assert(sd.size() == 3);
+                std::string solver = sd[0];
+                auto it = _solverDefaultOptions.find(solver);
+                if (it == _solverDefaultOptions.end()) {
+                  std::vector<std::string> solverOptions({sd[1], sd[2]});
+                  _solverDefaultOptions.insert(std::make_pair(solver, solverOptions));
+                } else {
+                  std::vector<std::string>& opts = it->second;
+                  bool found = false;
+                  for (unsigned int i = 0; i < opts.size(); i += 2) {
+                    if (opts[i] == sd[1]) {
+                      // Override existing option value
+                      opts[i + 1] = sd[2];
+                      found = true;
+                      break;
                     }
                   }
+                  if (!found) {
+                    // Option didn't exist, add to end of list
+                    opts.push_back(sd[1]);
+                    opts.push_back(sd[2]);
+                  }
                 }
-              } else {
-                throw ConfigException("invalid configuration item");
               }
-            } else {
-              throw ConfigException("invalid configuration item");
             }
           }
-        } else {
-          std::cerr << errstream.str();
-          throw ConfigException("internal error");
         }
       } catch (ConfigException& e) {
         log << "Warning: invalid configuration file: " << e.msg() << "\n";
       } catch (Exception& e) {
-        log << "Warning: invalid configuration file: " << e.what() << "\n";
+        log << "Warning: " << e.what() << " occured while reading " << cf << ": " << e.msg()
+            << "\n";
       }
     }
   }
@@ -665,7 +679,22 @@ SolverConfigs::SolverConfigs(std::ostream& log) {
   if (_mznlibDir != "/usr/share/minizinc" && FileUtils::directory_exists("/usr/share")) {
     _solverPath.emplace_back("/usr/share/minizinc/solvers");
   }
+  if (_mznlibDir != "/home/linuxbrew/.linuxbrew/share/minizinc" &&
+      FileUtils::directory_exists("/home/linuxbrew/.linuxbrew/share")) {
+    _solverPath.emplace_back("/home/linuxbrew/.linuxbrew/share/minizinc/solvers");
+  }
+  if (_mznlibDir != "/opt/homebrew/share/minizinc" &&
+      FileUtils::directory_exists("/opt/homebrew/share")) {
+    _solverPath.emplace_back("/opt/homebrew/share/minizinc/solvers");
+  }
 #endif
+
+  // Add share/minizinc/solvers next to current exe to solver search paths as for some builtin
+  // solvers we copy the mznlib and msc there to avoid keeping a copy in this repo
+  auto progpath_share = FileUtils::file_path(FileUtils::progpath() + "/share/minizinc");
+  if (progpath_share != _mznlibDir && FileUtils::directory_exists(progpath_share)) {
+    _solverPath.emplace_back(progpath_share + "/solvers");
+  }
 }
 
 void SolverConfigs::populate(std::ostream& log) {
@@ -726,15 +755,15 @@ std::string SolverConfigs::solverConfigsJSON() const {
   std::ostringstream oss;
 
   SortByName sortByName(_solvers);
-  std::vector<int> solversIdx(_solvers.size());
-  for (unsigned int i = 0; i < solversIdx.size(); i++) {
+  std::vector<size_t> solversIdx(_solvers.size());
+  for (size_t i = 0; i < solversIdx.size(); i++) {
     solversIdx[i] = i;
   }
   std::sort(solversIdx.begin(), solversIdx.end(), sortByName);
 
   bool hadSolver = false;
   oss << "[";
-  for (unsigned int i = 0; i < _solvers.size(); i++) {
+  for (size_t i = 0; i < _solvers.size(); i++) {
     const SolverConfig& sc = _solvers[solversIdx[i]];
     if (std::find(sc.tags().begin(), sc.tags().end(), "__internal__") != sc.tags().end()) {
       continue;
@@ -761,16 +790,14 @@ std::string get_version(const std::string& t) {
 }
 }  // namespace
 
-const SolverConfig& SolverConfigs::config(const std::string& _s) {
+SolverConfig& SolverConfigs::config(const std::string& _s) {
   std::string s;
   if (_s.size() > 4 && _s.substr(_s.size() - 4) == ".msc") {
     SolverConfig sc = SolverConfig::load(_s);
     addConfig(sc);
-    s = sc.id() + "@" + sc.version();
-  } else {
-    s = _s;
+    return _solvers.back();
   }
-  s = string_to_lower(s);
+  s = string_to_lower(_s);
   std::vector<std::string> tags;
   std::istringstream iss(s);
   std::string next_s;
