@@ -28,14 +28,12 @@
 #include <stdexcept>
 #include <string>
 
-#ifdef GUROBI_PLUGIN
-#ifdef HAS_DLFCN_H
-#include <dlfcn.h>
-#elif defined HAS_WINDOWS_H
+#ifdef _WIN32
 #define NOMINMAX  // Ensure the words min/max remain available
 #include <Windows.h>
 #undef ERROR
-#endif
+#else
+#include <dlfcn.h>
 #endif
 
 #include <minizinc/solvers/MIP/MIP_gurobi_wrap.hh>
@@ -63,7 +61,7 @@ string MIPGurobiWrapper::getVersion(FactoryOptions& factoryOpt,
     mgw.dll_GRBversion(&major, &minor, &technical);
     oss << major << '.' << minor << '.' << technical;
     return oss.str();
-  } catch (MiniZinc::InternalError&) {
+  } catch (MiniZinc::Error&) {
     return "<unknown version>";
   }
 }
@@ -74,7 +72,7 @@ vector<string> MIPGurobiWrapper::getRequiredFlags(FactoryOptions& f) {
   try {
     mgw.checkDLL();
     return {};
-  } catch (MiniZinc::InternalError&) {
+  } catch (MiniZinc::Error&) {
     return {"--gurobi-dll"};
   }
 }
@@ -91,13 +89,14 @@ vector<string> MIPGurobiWrapper::getStdFlags() { return {"-i", "-p", "-s", "-v"}
 
 vector<string> gurobi_dlls() {
   const vector<string> versions = {
-      "913", "912", "911", "910", "904",  // Potential future versions which should load correctly
-      "903", "902", "901", "900", "811", "810", "801", "800", "752",
-      "751", "750", "702", "701", "700", "652", "651", "650"};
+      "1013", "1012", "1011", "1010", "1003", "1002", "1001",  // Potential future versions which
+                                                               // should load correctly
+      "1000", "952", "951", "950", "912", "911", "910", "903", "902", "901", "900", "811", "810",
+      "801", "800", "752", "751", "750", "702", "701", "700", "652", "651", "650"};
   vector<string> dlls;
   string lastMajorVersion;
   for (const auto& version : versions) {
-    string majorVersion = version.substr(0, 2);
+    string majorVersion = version.substr(0, version.length() - 1);
     if (majorVersion != lastMajorVersion) {
       dlls.push_back("gurobi" + majorVersion);
       lastMajorVersion = majorVersion;
@@ -106,6 +105,8 @@ vector<string> gurobi_dlls() {
     dlls.push_back("C:\\gurobi" + version + "\\win64\\bin\\gurobi" + majorVersion + ".dll");
 #elif __APPLE__
     dlls.push_back("/Library/gurobi" + version + "/mac64/lib/libgurobi" + majorVersion + ".dylib");
+    dlls.push_back("/Library/gurobi" + version + "/macos_universal2/lib/libgurobi" + majorVersion +
+                   ".dylib");
 #else
     dlls.push_back("/opt/gurobi" + version + "/linux64/lib/libgurobi" + majorVersion + ".so");
 #endif
@@ -195,7 +196,8 @@ bool MIPGurobiWrapper::Options::processOption(int& i, std::vector<std::string>&
   std::string buf;
   if (cop.get("-i")) {
     flagIntermediate = true;
-  } else if (string(argv[i]) == "-f") {              // NOLINT: Allow repeated empty if
+  } else if (string(argv[i]) == "-f" ||
+             string(argv[i]) == "--free-search") {   // NOLINT: Allow repeated empty if
   } else if (string(argv[i]) == "--fixed-search") {  // NOLINT: Allow repeated empty if
     nFreeSearch = MIPGurobiWrapper::SearchType::FIXED_SEARCH;
   } else if (string(argv[i]) == "--uniform-search") {  // NOLINT: Allow repeated empty if
@@ -242,55 +244,48 @@ void MIPGurobiWrapper::wrapAssert(bool cond, const string& msg, bool fTerm) {
     }
     string msgAll =
         ("  MIPGurobiWrapper runtime error:  " + _gurobiBuffer + "\nMessage from caller: " + msg);
-    cerr << msgAll << "\nGurobi error code: " << _error << endl;
     if (fTerm) {
-      cerr << "TERMINATING." << endl;
-      throw runtime_error(msgAll);
+      throw MiniZinc::Error(msgAll);
     }
+    cerr << msgAll << "\nGurobi error code: " << _error << endl;
   }
 }
 
-#ifdef GUROBI_PLUGIN
-
 namespace {
 void* dll_open(const char* file) {
-#ifdef HAS_DLFCN_H
-  if (MiniZinc::FileUtils::is_absolute(file)) {
-    return dlopen(file, RTLD_NOW);
-  }
-  return dlopen((std::string("lib") + file + ".so").c_str(), RTLD_NOW);
-
-#else
+#ifdef _WIN32
   if (MiniZinc::FileUtils::is_absolute(file)) {
     return LoadLibrary(file);
   }
   return LoadLibrary((std::string(file) + ".dll").c_str());
+#else
+  if (MiniZinc::FileUtils::is_absolute(file)) {
+    return dlopen(file, RTLD_NOW);
+  }
+  return dlopen((std::string("lib") + file + ".so").c_str(), RTLD_NOW);
 #endif
 }
 void* dll_sym(void* dll, const char* sym) {
-#ifdef HAS_DLFCN_H
-  void* ret = dlsym(dll, sym);
-#else
+#ifdef _WIN32
   void* ret = GetProcAddress((HMODULE)dll, sym);
+#else
+  void* ret = dlsym(dll, sym);
 #endif
   if (ret == nullptr) {
-    throw MiniZinc::InternalError("cannot load symbol " + string(sym) + " from gurobi dll");
+    throw MiniZinc::Error("cannot load symbol " + string(sym) + " from gurobi dll");
   }
   return ret;
 }
 void dll_close(void* dll) {
-#ifdef HAS_DLFCN_H
-  dlclose(dll);
-#else
+#ifdef _WIN32
   FreeLibrary((HMODULE)dll);
+#else
+  dlclose(dll);
 #endif
 }
 }  // namespace
 
-#endif
-
 void MIPGurobiWrapper::checkDLL() {
-#ifdef GUROBI_PLUGIN
   _gurobiDll = nullptr;
   if (!_factoryOptions.gurobiDll.empty()) {
     _gurobiDll = dll_open(_factoryOptions.gurobiDll.c_str());
@@ -305,9 +300,9 @@ void MIPGurobiWrapper::checkDLL() {
 
   if (_gurobiDll == nullptr) {
     if (_factoryOptions.gurobiDll.empty()) {
-      throw MiniZinc::InternalError("cannot load gurobi dll, specify --gurobi-dll");
+      throw MiniZinc::Error("cannot load gurobi dll, specify --gurobi-dll");
     }
-    throw MiniZinc::InternalError("cannot load gurobi dll `" + _factoryOptions.gurobiDll + "'");
+    throw MiniZinc::Error("cannot load gurobi dll `" + _factoryOptions.gurobiDll + "'");
   }
 
   *(void**)(&dll_GRBversion) = dll_sym(_gurobiDll, "GRBversion");
@@ -351,48 +346,6 @@ void MIPGurobiWrapper::checkDLL() {
   *(void**)(&dll_GRBgetintparaminfo) = dll_sym(_gurobiDll, "GRBgetintparaminfo");
   *(void**)(&dll_GRBgetdblparaminfo) = dll_sym(_gurobiDll, "GRBgetdblparaminfo");
   *(void**)(&dll_GRBgetstrparaminfo) = dll_sym(_gurobiDll, "GRBgetstrparaminfo");
-
-#else
-
-  dll_GRBversion = GRBversion;
-  dll_GRBaddconstr = GRBaddconstr;
-  dll_GRBaddgenconstrIndicator = GRBaddgenconstrIndicator;
-  dll_GRBaddvars = GRBaddvars;
-  dll_GRBcbcut = GRBcbcut;
-  dll_GRBcbget = GRBcbget;
-  dll_GRBcblazy = GRBcblazy;
-  dll_GRBfreeenv = GRBfreeenv;
-  dll_GRBfreemodel = GRBfreemodel;
-  dll_GRBgetdblattr = GRBgetdblattr;
-  dll_GRBgetdblattrarray = GRBgetdblattrarray;
-  dll_GRBgetenv = GRBgetenv;
-  dll_GRBgeterrormsg = GRBgeterrormsg;
-  dll_GRBgetintattr = GRBgetintattr;
-  dll_GRBloadenv = GRBloadenv;
-  dll_GRBnewmodel = GRBnewmodel;
-  dll_GRBoptimize = GRBoptimize;
-  dll_GRBreadparams = GRBreadparams;
-  dll_GRBsetcallbackfunc = GRBsetcallbackfunc;
-  dll_GRBsetdblparam = GRBsetdblparam;
-  dll_GRBsetintattr = GRBsetintattr;
-  dll_GRBsetintattrlist = GRBsetintattrlist;
-  dll_GRBsetdblattrelement = GRBsetdblattrelement;
-  dll_GRBsetdblattrlist = GRBsetdblattrlist;
-  dll_GRBsetintparam = GRBsetintparam;
-  dll_GRBsetstrparam = GRBsetstrparam;
-  dll_GRBterminate = GRBterminate;
-  dll_GRBupdatemodel = GRBupdatemodel;
-  dll_GRBwrite = GRBwrite;
-  dll_GRBwriteparams = GRBwriteparams;
-  dll_GRBemptyenv = GRBemptyenv;
-  dll_GRBgetnumparams = GRBgetnumparams;
-  dll_GRBgetparamname = GRBgetparamname;
-  dll_GRBgetparamtype = GRBgetparamtype;
-  dll_GRBgetintparaminfo = dll_GRBgetintparaminfo;
-  dll_GRBgetdblparaminfo = dll_GRBgetdblparaminfo;
-  dll_GRBgetstrparaminfo = dll_GRBgetstrparaminfo;
-
-#endif
 }
 
 void MIPGurobiWrapper::openGUROBI() {
@@ -430,10 +383,8 @@ void MIPGurobiWrapper::closeGUROBI() {
     dll_GRBfreeenv(_env);
   }
   /// and at last:
-//   MIPWrapper::cleanup();
-#ifdef GUROBI_PLUGIN
+  //   MIPWrapper::cleanup();
   // dll_close(_gurobiDll);    // Is called too many times, disabling. 2019-05-06
-#endif
 }
 
 std::vector<MiniZinc::SolverConfig::ExtraFlag> MIPGurobiWrapper::getExtraFlags(
@@ -461,7 +412,10 @@ std::vector<MiniZinc::SolverConfig::ExtraFlag> MIPGurobiWrapper::getExtraFlags(
           param_name == GRB_DBL_PAR_NODEFILESTART || param_name == GRB_STR_PAR_NODEFILEDIR ||
           param_name == GRB_DBL_PAR_MIPGAPABS || param_name == GRB_INT_PAR_MIPFOCUS ||
           param_name == GRB_DBL_PAR_MIPGAP || param_name == GRB_DBL_PAR_INTFEASTOL ||
-          param_name == GRB_DBL_PAR_FEASIBILITYTOL || param_name == GRB_INT_PAR_NONCONVEX ||
+          param_name == GRB_DBL_PAR_FEASIBILITYTOL ||
+#ifdef GRB_INT_PAR_NONCONVEX
+          param_name == GRB_INT_PAR_NONCONVEX ||
+#endif
           param_name == GRB_INT_PAR_PRECRUSH || param_name == GRB_INT_PAR_LAZYCONSTRAINTS ||
           param_name == GRB_STR_PAR_DUMMY) {
         // These parameters are handled by us or are not useful
@@ -507,7 +461,7 @@ std::vector<MiniZinc::SolverConfig::ExtraFlag> MIPGurobiWrapper::getExtraFlags(
                          param_default);
     }
     return flags;
-  } catch (MiniZinc::InternalError&) {
+  } catch (MiniZinc::Error&) {
     return {};
   }
   return {};
@@ -531,11 +485,11 @@ void MIPGurobiWrapper::doAddVars(size_t n, double* obj, double* lb, double* ub,
         ctype[i] = GRB_BINARY;
         break;
       default:
-        throw runtime_error("  MIPWrapper: unknown variable type");
+        throw MiniZinc::InternalError("  MIPWrapper: unknown variable type");
     }
   }
   _error = dll_GRBaddvars(_model, static_cast<int>(n), 0, nullptr, nullptr, nullptr, obj, lb, ub,
-                          &ctype[0], &pcNames[0]);
+                          ctype.data(), pcNames.data());
   wrapAssert(_error == 0, "Failed to declare variables.");
   _error = dll_GRBupdatemodel(_model);
   wrapAssert(_error == 0, "Failed to update model.");
@@ -550,7 +504,7 @@ static char get_grb_sense(MIPWrapper::LinConType s) {
     case MIPWrapper::GQ:
       return GRB_GREATER_EQUAL;
     default:
-      throw runtime_error("  MIPGurobiWrapper: unknown constraint sense");
+      throw MiniZinc::InternalError("  MIPGurobiWrapper: unknown constraint sense");
   }
 }
 
@@ -760,6 +714,7 @@ static int __stdcall solcallback(GRBmodel* model, void* cbdata, int where, void*
       /// Call the user function:
       if (info->solcbfn != nullptr) {
         (*info->solcbfn)(*info->pOutput, info->psi);
+        info->printed = true;
       }
 
       if (0 == info->nTimeoutFeas) {
@@ -785,7 +740,7 @@ static int __stdcall solcallback(GRBmodel* model, void* cbdata, int where, void*
       //         cerr << "\n   N CUTS:  " << nCuts << endl;
       for (auto& cd : cutInput) {
         if ((cd.mask & (MIPWrapper::MaskConsType_Usercut | MIPWrapper::MaskConsType_Lazy)) == 0) {
-          throw runtime_error("Cut callback: should be user/lazy");
+          throw MiniZinc::InternalError("Cut callback: should be user/lazy");
         }
         if ((cd.mask & MIPWrapper::MaskConsType_Usercut) != 0) {
           int _error =
@@ -962,7 +917,7 @@ void MIPGurobiWrapper::solve() {        // Move into ancestor?
   /// Solution callback
   output.nCols = static_cast<int>(colObj.size());
   _x.resize(output.nCols);
-  output.x = &_x[0];
+  output.x = _x.data();
   SolCallbackFn solcbfn = cbui.solcbfn;
   if (true) {  // NOLINT: Need for logging
     cbui.fVerb = fVerbose;
@@ -1039,7 +994,7 @@ void MIPGurobiWrapper::solve() {        // Move into ancestor?
   }
 
   cbui.pOutput->dWallTime0 = output.dWallTime0 = std::chrono::steady_clock::now();
-  output.dCPUTime = cbui.pOutput->cCPUTime0 = std::clock();
+  output.dCPUTime = static_cast<double>(cbui.pOutput->cCPUTime0 = std::clock());
 
   /* Optimize the problem and obtain solution. */
   _error = dll_GRBoptimize(_model);
@@ -1047,7 +1002,7 @@ void MIPGurobiWrapper::solve() {        // Move into ancestor?
 
   output.dWallTime =
       std::chrono::duration<double>(std::chrono::steady_clock::now() - output.dWallTime0).count();
-  output.dCPUTime = (std::clock() - output.dCPUTime) / CLOCKS_PER_SEC;
+  output.dCPUTime = (static_cast<double>(std::clock()) - output.dCPUTime) / CLOCKS_PER_SEC;
 
   int solstat;
   _error = dll_GRBgetintattr(_model, GRB_INT_ATTR_STATUS, &solstat);
@@ -1064,10 +1019,10 @@ void MIPGurobiWrapper::solve() {        // Move into ancestor?
     assert(cur_numcols == colObj.size());
 
     _x.resize(cur_numcols);
-    output.x = &_x[0];
+    output.x = _x.data();
     _error = dll_GRBgetdblattrarray(_model, GRB_DBL_ATTR_X, 0, cur_numcols, (double*)output.x);
     wrapAssert(_error == 0, "Failed to get variable values.");
-    if (!_options->flagIntermediate && (solcbfn != nullptr)) {
+    if ((!_options->flagIntermediate || !cbui.printed) && (solcbfn != nullptr)) {
       solcbfn(output, cbui.psi);
     }
   }
@@ -1076,9 +1031,8 @@ void MIPGurobiWrapper::solve() {        // Move into ancestor?
                          : std::numeric_limits<double>::max();
   int nObj = 0;
   dll_GRBgetintattr(_model, GRB_INT_ATTR_NUMOBJ, &nObj);
-  if (1 >= nObj) {
-    _error = dll_GRBgetdblattr(_model, GRB_DBL_ATTR_OBJBOUNDC, &output.bestBound);
-    wrapAssert(_error == 0, "Failed to get the best bound.", false);
+  if (1 >= nObj) {  // This can fail, e.g., for LP or QP
+    dll_GRBgetdblattr(_model, GRB_DBL_ATTR_OBJBOUNDC, &output.bestBound);
   }
   double nNodes = -1;
   _error = dll_GRBgetdblattr(_model, GRB_DBL_ATTR_NODECOUNT, &nNodes);
