@@ -110,7 +110,8 @@ bool MIPosicbcWrapper::Options::processOption(int& i, std::vector<std::string>&
   std::string buffer;
   if (cop.get("-i")) {
     flagIntermediate = true;
-  } else if (string(argv[i]) == "-f") {  // NOLINT: Allow repeated empty if
+  } else if (string(argv[i]) == "-f" ||
+             string(argv[i]) == "--free-search") {  // NOLINT: Allow repeated empty if
     //     std::cerr << "  Flag -f: ignoring fixed strategy anyway." << std::endl;
   } else if (cop.get("--writeModel", &buffer)) {
     sExportModel = MiniZinc::FileUtils::file_path(buffer, workingDir);
@@ -216,16 +217,10 @@ std::vector<MiniZinc::SolverConfig::ExtraFlag> MIPosicbcWrapper::getExtraFlags(
   return res;
 }
 
-void MIPosicbcWrapper::wrapAssert(bool cond, const string& msg, bool fTerm) {
+void MIPosicbcWrapper::wrapAssert(bool cond, const string& msg) {
   if (!cond) {
-    //       strcpy(_osicbcBuffer, "[NO ERROR STRING GIVEN]");
-    //       CBCgeterrorstring (env, status, _osicbcBuffer);
     string msgAll = ("  MIPosicbcWrapper runtime error:  " + msg + "  " + _osicbcBuffer);
-    cerr << msgAll << endl;
-    if (fTerm) {
-      cerr << "TERMINATING." << endl;
-      throw runtime_error(msgAll);
-    }
+    throw MiniZinc::Error(msgAll);
   }
 }
 
@@ -236,8 +231,8 @@ void MIPosicbcWrapper::doAddVars(size_t n, double* obj, double* lb, double* ub,
   //   vector<char*> pcNames(n);
   CoinPackedVector cpv;
   vector<CoinPackedVectorBase*> pCpv(n, &cpv);
-  _osi.addCols(n, pCpv.data(), lb, ub, obj);  // setting integer & names later
-  //   status = CBCnewcols (env, lp, n, obj, lb, ub, &ctype[0], &pcNames[0]);
+  _osi.addCols(static_cast<int>(n), pCpv.data(), lb, ub, obj);  // setting integer & names later
+  //   status = CBCnewcols (env, lp, n, obj, lb, ub, ctype.data(), pcNames.data());
   //   wrapAssert( !status,  "Failed to declare variables." );
 }
 
@@ -256,7 +251,7 @@ void MIPosicbcWrapper::addRow(int nnz, int* rmatind, double* rmatval, MIPWrapper
       rub = _osi.getInfinity();
       break;
     default:
-      throw runtime_error("  MIPWrapper: unknown constraint type");
+      throw MiniZinc::InternalError("  MIPWrapper: unknown constraint type");
   }
   // ignoring mask for now.  TODO
   // 1-by-1 too slow:
@@ -370,7 +365,7 @@ static int statusOfCbc = 0;
 static CoinSighandler_t saveSignal = static_cast<CoinSighandler_t>(nullptr);
 
 extern "C" {
-static void
+void
 #if defined(_MSC_VER)
     __cdecl
 #endif  // _MSC_VER
@@ -482,7 +477,10 @@ CbcEventHandler::CbcAction MyEventHandler3::event(CbcEvent whichEvent) {
         // If preprocessing was done solution will be to processed model
         //       int numberColumns = model_->getNumCols();
         const double* bestSolution = model_->bestSolution();
-        assert(bestSolution);
+        if (bestSolution == nullptr) {
+          // Could happen if we only have heuristic solution?
+          return noAction;
+        }
         //       printf("value of solution is %g\n",model_->getObjValue());
 
         // Trying to obtain solution for the original model:
@@ -643,7 +641,6 @@ MIPosicbcWrapper::Status MIPosicbcWrapper::convertStatus(CbcModel* pModel) {
   if (pModel->isProvenOptimal()) {
     s = Status::OPT;
     output.statusName = "Optimal";
-    //        wrapAssert(_osi., "Optimality reported but pool empty?", false);
   } else if (pModel->isProvenInfeasible()) {
     s = Status::UNSAT;
     output.statusName = "Infeasible";
@@ -656,7 +653,7 @@ MIPosicbcWrapper::Status MIPosicbcWrapper::convertStatus(CbcModel* pModel) {
     s = Status::SAT;
     output.statusName = "Feasible";
   } else if (pModel->isAbandoned()) {  // AFTER feas-ty
-    s = Status::__ERROR;
+    s = Status::ERROR_STATUS;
     output.statusName = "Abandoned";
   } else {
     s = Status::UNKNOWN;
@@ -671,7 +668,6 @@ MIPosicbcWrapper::Status MIPosicbcWrapper::convertStatus() {
   if (_osi.isProvenOptimal()) {
     s = Status::OPT;
     output.statusName = "Optimal";
-    //        wrapAssert(_osi., "Optimality reported but pool empty?", false);
   } else if (_osi.isProvenPrimalInfeasible()) {
     s = Status::UNSAT;
     output.statusName = "Infeasible";
@@ -680,7 +676,7 @@ MIPosicbcWrapper::Status MIPosicbcWrapper::convertStatus() {
     output.statusName = "Dual infeasible";
     //        s = Status::UNSATorUNBND;
   } else if (_osi.isAbandoned()) {
-    s = Status::__ERROR;
+    s = Status::ERROR_STATUS;
     output.statusName = "Abandoned";
   } else if  // wrong: (pModel->getColSolution())
       (fabs(_osi.getObjValue()) < _osi.getInfinity()) {
@@ -709,7 +705,7 @@ void MIPosicbcWrapper::solve() {  // Move into ancestor?
     for (int i = 0; i < _rowlb.size(); ++i) {
       pRows[i] = &_rows[i];
     }
-    _osi.addRows(_rowlb.size(), pRows.data(), _rowlb.data(), _rowub.data());
+    _osi.addRows(static_cast<int>(_rowlb.size()), pRows.data(), _rowlb.data(), _rowub.data());
     //     rowStarts.clear();
     //     columns.clear();
     //     element.clear();
@@ -728,10 +724,10 @@ void MIPosicbcWrapper::solve() {  // Move into ancestor?
         if (REAL != colTypes[i]
             //           && is_used[i]
         ) {
-          integer_vars.push_back(i);
+          integer_vars.push_back(static_cast<int>(i));
         }
       }
-      _osi.setInteger(integer_vars.data(), integer_vars.size());
+      _osi.setInteger(integer_vars.data(), static_cast<int>(integer_vars.size()));
     }
     if (!_options->sExportModel.empty()) {
       // Not implemented for OsiClp:
@@ -763,12 +759,12 @@ void MIPosicbcWrapper::solve() {  // Move into ancestor?
       cerr << " Model creation..." << endl;
     }
 
-    // #define __USE_CbcSolver__  -- not linked rev2274
+    // #define MZN_USE_CbcSolver  -- not linked rev2274
     /// FOR WARMSTART
     for (const auto& vv : _warmstart) {
       _osi.setColName(vv.first, colNames[vv.first]);
     }
-#ifdef __USE_CbcSolver__
+#ifdef MZN_USE_CbcSolver
     CbcSolver control(osi);
     // initialize
     control.fillValuesInSolver();
@@ -841,9 +837,9 @@ void MIPosicbcWrapper::solve() {  // Move into ancestor?
     // Turn off OSICBC logging
 
     /// Solution callback
-    output.nCols = colObj.size();
+    output.nCols = static_cast<int>(colObj.size());
     //    x.resize(output.nCols);
-    //    output.x = &x[0];
+    //    output.x = x.data();
 
     if (_options->flagIntermediate && (cbui.solcbfn != nullptr)) {
       // Event handler. Should be after CbcMain0()?
@@ -894,7 +890,7 @@ void MIPosicbcWrapper::solve() {  // Move into ancestor?
                 rc.setLb(cut.rhs);
                 rc.setUb(cut.rhs);
             }
-            rc.setRow(cut.rmatind.size(), cut.rmatind.data(), cut.rmatval.data());
+            rc.setRow(static_cast<int>(cut.rmatind.size()), cut.rmatind.data(), cut.rmatval.data());
             cs.insertIfNotDuplicate(rc);
           }
         }
@@ -927,7 +923,7 @@ void MIPosicbcWrapper::solve() {  // Move into ancestor?
     }
 
     cbui.pOutput->dWallTime0 = output.dWallTime0 = std::chrono::steady_clock::now();
-    output.dCPUTime = clock();
+    output.dCPUTime = static_cast<double>(clock());
 
     /* OLD: Optimize the problem and obtain solution. */
     //       model.branchAndBound();
@@ -938,14 +934,14 @@ void MIPosicbcWrapper::solve() {  // Move into ancestor?
 //      CbcCbcParamUtils::setCbcModelDefaults(model) ;
 //       const char * argv2[]={"mzn-cbc","-solve","-quit"};
 //        CbcMain1(3,argv2,model);
-#ifdef __USE_CbcSolver__
+#ifdef MZN_USE_CbcSolver
     if (fVerbose)
       cerr << "  Calling control.solve() with options '" << options->cbcCmdOptions << "'..."
            << endl;
     control.solve(options->cbcCmdOptions.c_str(), 1);
 #else
-#define __USE_callCbc1__
-#ifdef __USE_callCbc1__
+#define MZN_USE_callCbc1
+#ifdef MZN_USE_callCbc1
     if (fVerbose) {
       cerr << "  Calling CbcMain with command 'cbc";
       for (const auto& arg : _options->cbcCmdOptions) {
@@ -953,7 +949,7 @@ void MIPosicbcWrapper::solve() {  // Move into ancestor?
       }
       cerr << "'..." << endl;
     }
-    CbcMain(cbc_argc, &cbc_argv[0], model);
+    CbcMain(static_cast<int>(cbc_argc), cbc_argv.data(), model);
     // callCbc(_options->cbcCmdOptions, model);
 //     callCbc1(cbcCmdOptions, model, callBack);
 // What is callBack() for?    TODO
@@ -979,7 +975,7 @@ void MIPosicbcWrapper::solve() {  // Move into ancestor?
 
     output.dWallTime =
         std::chrono::duration<double>(std::chrono::steady_clock::now() - output.dWallTime0).count();
-    output.dCPUTime = (clock() - output.dCPUTime) / CLOCKS_PER_SEC;
+    output.dCPUTime = (static_cast<double>(clock()) - output.dCPUTime) / CLOCKS_PER_SEC;
 
     output.status = convertStatus(&model);
     //    output.status = convertStatus();
